Package: bpca
Version: 1.5-0
Date: 2026-05-25
Title: Biplot Analysis for Multivariate Data Using Principal Components
Authors@R: c(person("José C.", "Faria", role = c("aut", "cre"),
               email = "joseclaudio.faria@gmail.com"),
             person("Ivan B.", "Allaman", role = "aut"),
             person("Clarice G. B.", "Demétrio", role = "aut"))
Maintainer: José C. Faria <joseclaudio.faria@gmail.com>
Depends: R (>= 4.1.0)
Imports: scatterplot3d, rgl, xtable
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
Description: Provides tools for creating 2D and 3D biplots of multivariate data
    based on principal components analysis, together with diagnostics for
    reduction quality and enhanced visualization of variables and objects.
License: GPL-2
URL: https://github.com/jcfaria/bpca
BugReports: https://github.com/jcfaria/bpca/issues
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-25 16:33:37 UTC; jcfaria
Author: José C. Faria [aut, cre],
  Ivan B. Allaman [aut],
  Clarice G. B. Demétrio [aut]
Repository: CRAN
Date/Publication: 2026-05-25 17:30:02 UTC
Built: R 4.6.0; ; 2026-05-25 18:25:04 UTC; unix

Package: bpca
Version: 1.5-0
Date: 2026-05-25
Title: Biplot Analysis for Multivariate Data Using Principal Components
Authors@R: c(person("José C.", "Faria", role = c("aut", "cre"),
               email = "joseclaudio.faria@gmail.com"),
             person("Ivan B.", "Allaman", role = "aut"),
             person("Clarice G. B.", "Demétrio", role = "aut"))
Maintainer: José C. Faria <joseclaudio.faria@gmail.com>
Depends: R (>= 4.1.0)
Imports: scatterplot3d, rgl, xtable
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
Description: Provides tools for creating 2D and 3D biplots of multivariate data
    based on principal components analysis, together with diagnostics for
    reduction quality and enhanced visualization of variables and objects.
License: GPL-2
URL: https://github.com/jcfaria/bpca
BugReports: https://github.com/jcfaria/bpca/issues
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-25 16:33:37 UTC; jcfaria
Author: José C. Faria [aut, cre],
  Ivan B. Allaman [aut],
  Clarice G. B. Demétrio [aut]
Repository: CRAN
Date/Publication: 2026-05-25 17:30:02 UTC
Built: R 4.6.0; ; 2026-05-25 17:45:10 UTC; unix

# Exports
export(bpca)
export(dt.tools)
export(qbpca)
export(var.rbf)
export(var.rdf)
export(xtable.bpca)

# Imports
importFrom(graphics, abline, arrows, axis, box, identify, legend, lines, par, plot, points, polygon, rect, segments, strheight, strwidth, symbols, text, title)
importFrom(grDevices, chull, devAskNewPage, dev.interactive, dev.new, gray, rainbow)
importFrom(rgl, abclines3d, axes3d, clear3d, decorate3d, plot3d, segments3d, spheres3d, text3d, title3d)
importFrom(scatterplot3d, scatterplot3d)
importFrom(stats, cor, prcomp, sd)
importFrom(utils, combn)
importFrom(xtable, print.xtable, xtable)

# S3 methods
S3method(bpca,default)
S3method(bpca,prcomp)
S3method(plot,bpca.2d)
S3method(plot,bpca.3d)
S3method(plot,qbpca)
S3method(print,summary.bpca)
S3method(print,var.rdf)
S3method(print,xtable.bpca)
S3method(summary,bpca)
S3method(xtable,bpca)

<!-- José Cláudio Faria -->
# bpca

<!-- Badges -->
[![CRAN status](https://www.r-pkg.org/badges/version/bpca)](https://cran.r-project.org/package=bpca)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/bpca)](https://cran.r-project.org/package=bpca)
[![CRAN checks](https://badges.cranchecks.info/worst/bpca.svg)](https://cran.r-project.org/web/checks/check_results_bpca.html)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![License: GPL-2](https://img.shields.io/badge/License-GPL--2-blue.svg)](https://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html)

`bpca` is an R package for biplot analysis based on principal components.

## Key Features

- **PCA-based Biplot analysis** in reduced-dimensional spaces (2D and 3D).
- **Multiple factorization methods** (`hj`, `sqrt`, `jk`, `gh`) for different interpretations.
- **Quality diagnostics** for dimensionality reduction and representation fidelity.
- **Reporting support** with `xtable(bpca(...))` and `print(..., type = "html")`.

## Installation

Install from CRAN:

```r
install.packages("bpca")
```

Install the development version from GitHub:

```r
# install.packages("remotes")
remotes::install_github("jcfaria/bpca")
```

## Quick Start

```r
library(bpca)

# Example with the classic iris dataset
bp <- bpca(iris[-5], d = 1:2)
plot(
  bp,
  var.col = "blue",
  var.factor = 2,
  main = "Biplot - Iris Dataset"
)
```

For more complete examples, see:

- `demo("bpca", package = "bpca")`
- `vignette("bpca-overview", package = "bpca")`

## Project Layout

- `/R`: Core computational and plotting functions.
- `/data`: Example datasets shipped with the package.
- `/demo`: Runnable demos illustrating usage.
- `/man`: Documentation (`.Rd` files).
- `/inst`: Extra package materials (e.g. citations).
- `/vignettes`: Vignettes and tutorials.

## Contributing

Contributions are welcome. Open an **[Issue](https://github.com/jcfaria/bpca/issues)** or submit a **Pull Request** on [github.com/jcfaria/bpca](https://github.com/jcfaria/bpca) with:

- Bug fixes and performance improvements.
- Documentation and usability improvements.
- New ideas for diagnostics and visualization workflows.

## Roadmap

- Expand test coverage for edge cases and plotting behavior.
- Add practical vignettes with real-world datasets.
- Improve CI signals and package quality checks.

---
Developed by:  
Faria, J. C.; Allaman, I. B.  
Universidade Estadual de Santa Cruz - UESC  
Departamento de Ciências Exatas - DCEX  
Ilhéus - Bahia - Brasil  

Demétrio, C. G. B.  
Universidade de São Paulo - USP  
Escola Superior de Agricultura Luiz de Queiroz - ESALQ  
Piracicaba - São Paulo - Brasil

##
## iris demo: grouping objects with different symbols and colors (2D and 3D)
##

oask <- devAskNewPage(dev.interactive(orNone = TRUE))

# 2D
plot(bpca(iris[-5]),
  var.factor = .3,
  var.cex = .7,
  obj.names = FALSE,
  obj.cex = 1.5,
  obj.col = c(
    "red",
    "green3",
    "blue"
  )[as.numeric(iris$Species)],
  obj.pch = c(
    "+",
    "*",
    "-"
  )[as.numeric(iris$Species)]
)

# 3D static
plot(
  bpca(iris[-5],
    d = 1:3
  ),
  var.factor = .2,
  var.color = c("blue", "red"),
  var.cex = 1,
  obj.names = FALSE,
  obj.cex = 1,
  obj.col = c(
    "red",
    "green3",
    "blue"
  )[as.numeric(iris$Species)],
  obj.pch = c(
    "+",
    "*",
    "-"
  )[as.numeric(iris$Species)]
)

devAskNewPage(oask)

# 3D dynamic
plot(
  bpca(iris[-5],
    d = 1:3,
    method = "hj"
  ),
  rgl.use = TRUE,
  var.color = "brown",
  var.factor = .3,
  var.cex = 1.2,
  obj.names = FALSE,
  obj.cex = .5,
  obj.col = c(
    "red",
    "green3",
    "orange"
  )[as.numeric(iris$Species)],
  simple.axes = TRUE
)

### R code from vignette source 'latex-bpca.Rnw'

###################################################
### code chunk number 1: latex-bpca.Rnw:72-81
###################################################
library(bpca)
library(xtable)

bp <- bpca(iris[-5])

## The simplest possible
xtable(bp)

print(xtable(bp))


###################################################
### code chunk number 2: latex-bpca.Rnw:89-94
###################################################
## With caption and label
## It will use the methods print.xtable.bpca provided by the bpca package
xtable(bpca(iris[-5]),
       caption='Biplot of iris data (packages:datasets).',
       label='tbl_iris')


###################################################
### code chunk number 3: latex-bpca.Rnw:98-102
###################################################
## With caption and label
xtable(bpca(gabriel1971),
       caption='Biplot of gabriel1971 data (package:datasets).',
       label='tbl_gabriel')


###################################################
### code chunk number 4: latex-bpca.Rnw:110-123
###################################################
## With bold in the columns
bp_rock_x <- xtable(bpca(rock),
                    caption='Biplot of rock data (package:dtasets).',
                    label='tbl_rock')

bold <- function(x){
  paste('\\textbf{',
        x,
        '}')
}

print(bp_rock_x,
      sanitize.colnames.function = bold)


###################################################
### code chunk number 5: latex-bpca.Rnw:129-143
###################################################
## With italic in the rows
bp_USA_x <- xtable(bpca(USArrests),
                   caption='Biplot of USArrests data (package:datasets).',
                   label='tbl_USArrests')

italic <- function(x){
  paste('\\textit{',
        x,
        '}',
        sep='')
}

print(bp_USA_x,
      sanitize.rownames.function = italic)


###################################################
### code chunk number 6: latex-bpca.Rnw:150-175
###################################################
## Principal labels in portuguese
tbl_rock_x <- xtable(bpca(rock),
                     caption='Biplot of rock data (package:datasets).',
                     label='tbl_rock_2')

rownames(tbl_rock_x) <- gsub('Eigenvalues',
                             'Autovalores',
                             rownames(tbl_rock_x))

rownames(tbl_rock_x) <- gsub('Eigenvectors',
                             'Autovetores',
                             rownames(tbl_rock_x))

rownames(tbl_rock_x) <- gsub('Variance retained',
                             'Variância retida',
                             rownames(tbl_rock_x))

rownames(tbl_rock_x) <- gsub('Variance accumulated',
                             'Variância acumulada',
                             rownames(tbl_rock_x))

colnames(tbl_rock_x) <- c('CP1',
                          'CP2')

print(tbl_rock_x)


###################################################
### code chunk number 7: latex-bpca.Rnw:182-200
###################################################
## If you don't want to use the bpca formatting standard (method print.xtable.bpca),
## you can directly call the print.xtable function and format the table as you wish.
italic <- function(x){
  paste('\\textit{',
        x,
        '}',
        sep='')
}

print.xtable(xtable(bpca(rock),
             caption='Call directly the print.xtable function',
             label='tbl_directly'),
             sanitize.colnames.function=bold,
             sanitize.rownames.function=italic)

## To others formatations see:
## - ?xtable
## - ?print.xtable

##
## Diagnostic workflow for var.rd using the rock dataset
##

oask <- devAskNewPage(dev.interactive(orNone = TRUE))

bp <- bpca(rock,
  var.rb = TRUE,
  var.rd = TRUE
)
summary(bp)

# The total variance explained is satisfactory (>= .80)!

plot(bp)

# A more accurate diagnostic
bp$var.rd

# It is possible to observe that the variable 'perm'
# does not have good representation in 2D (bpca.2d).

# Observed correlations:
cor(rock)

# Projected correlations:
bp$var.rb

# Additional diagnostic
plot(
  qbpca(
    rock,
    bp
  ),
  highlight.width = 0.2
)

# This variable remains as important in a dimension not contemplated
# by the biplot reduction (PC3):

bp$eigenvectors

bp1 <- bpca(rock,
  d = 3:4
)

summary(bp1)

plot(bp1)

# Since this variable has poor representation in 2D, the recommendation is:
# 1- Avoid discussing it;
# 2- Consider incorporating that information with a 3D biplot (bpca.3d).

bp3 <- bpca(rock,
  d = 1:3,
  var.rb = TRUE,
  var.rd = TRUE
)

summary(bp3)

plot(bp3) # Static

plot(bp3,
  rgl.use = TRUE
) # Dynamic

bp3$var.rd # Nice!

# Additional diagnostic
plot(
  qbpca(
    rock,
    bp3
  ),
  highlight.width = 0.2
)

devAskNewPage(oask)

##
## Diagnostic of iris representation with `var.rd` parameter (2D and 3D)
##

oask <- devAskNewPage(dev.interactive(orNone = TRUE))

bp2 <- bpca(iris[-5],
  var.rb = TRUE,
  var.rd = TRUE,
  limit = 3
)

plot(bp2,
  obj.names = FALSE,
  obj.pch = c(
    "+",
    "-",
    "*"
  )[unclass(iris$Species)],
  obj.col = c(
    "red",
    "green3",
    "blue"
  )[unclass(iris$Species)],
  obj.cex = 1
)

bp2$var.rd
bp2$eigenvectors

# Graphical diagnostic
plot(
  bpca(iris[-5],
    d = 3:4
  ),
  obj.names = FALSE,
  obj.pch = c(
    "+",
    "-",
    "*"
  )[unclass(iris$Species)],
  obj.col = c(
    "red",
    "green3",
    "blue"
  )[unclass(iris$Species)],
  obj.cex = 1
)

# Interpretation:
# Sepal.Length followed by Petal.Width contains information in dimensions
# (PC3 is essentially a contrast between both) that was not fully
# captured by the biplot reduction (PC1 and PC2).
# Therefore, among all variables, they have a poor representation in 2D
# biplot.

bp3 <- bpca(iris[-5],
  d = 1:3,
  var.rb = TRUE,
  var.rd = TRUE,
  limit = 2
)

plot(bp3,
  obj.names = FALSE,
  obj.pch = c(
    "+",
    "-",
    "*"
  )[unclass(iris$Species)],
  obj.col = c(
    "red",
    "green3",
    "blue"
  )[unclass(iris$Species)],
  obj.cex = 1
)

bp3$var.rd
bp3$eigenvectors

round(
  bp3$var.rb,
  2
)

round(
  cor(iris[-5]),
  2
)

# Good representation of all variables with a 3D biplot.

devAskNewPage(oask)

##
## 2D quality comparison using var.rb + qbpca
##

oask <- devAskNewPage(dev.interactive(orNone = TRUE))

## Differences between methods of factorization
# SQRT
bp_sqrt <- bpca(gabriel1971,
  method = "sqrt",
  var.rb = TRUE
)

qbp_sqrt <- qbpca(
  gabriel1971,
  bp_sqrt
)

plot(qbp_sqrt,
  main = "sqrt - 2D \n (poor)"
)

# JK
bp_jk <- bpca(gabriel1971,
  method = "jk",
  var.rb = TRUE
)

qbp_jk <- qbpca(
  gabriel1971,
  bp_jk
)

plot(qbp_jk,
  main = "jk - 2D \n (very poor)"
)

# HJ
bp_hj <- bpca(gabriel1971,
  method = "hj",
  var.rb = TRUE
)

qbp_hj <- qbpca(
  gabriel1971,
  bp_hj
)

plot(qbp_hj,
  main = "hj - 2D \n (good)"
)

# GH
bp_gh <- bpca(gabriel1971,
  method = "gh",
  var.rb = TRUE
)

qbp_gh <- qbpca(
  gabriel1971,
  bp_gh
)

plot(qbp_gh,
  main = "gh - 2D \n (good)"
)

devAskNewPage(oask)

##
## 2D quality comparison using var.rb + qbpca
##

oask <- devAskNewPage(dev.interactive(orNone=TRUE))

## Differences between methods of factorization
# SQRT
bp_sqrt <- bpca(gabriel1971,
                method='sqrt',
                var.rb=TRUE)

qbp_sqrt <- qbpca(gabriel1971,
                  bp_sqrt)

plot(qbp_sqrt,
     main='sqrt - 2D \n (poor)')

# JK
bp_jk <- bpca(gabriel1971,
              method='jk',
              var.rb=TRUE)

qbp_jk <- qbpca(gabriel1971,
                bp_jk)

plot(qbp_jk,
     main='jk - 2D \n (very poor)')

# HJ
bp_hj <- bpca(gabriel1971,
              method='hj',
              var.rb=TRUE)

qbp_hj <- qbpca(gabriel1971,
                bp_hj)

plot(qbp_hj,
     main='hj - 2D \n (good)')

# GH
bp_gh <- bpca(gabriel1971,
              method='gh',
              var.rb=TRUE)

qbp_gh <- qbpca(gabriel1971,
                bp_gh)

plot(qbp_gh,
     main='gh - 2D \n (good)')

devAskNewPage(oask)

##
## Static 3D bpca workflow with scatterplot3d
##

oask <- devAskNewPage(dev.interactive(orNone = TRUE))

bp <- bpca(gabriel1971,
  d = 1:3
)

plot(bp)

# Explore the object created by bpca()
class(bp)
names(bp)
str(bp)

summary(bp)
bp$call
bp$eigenvalues
bp$eigenvectors
bp$number
bp$importance
bp$coord
bp$coord$objects
bp$coord$variables
bp$var.rb
bp$var.rd

# Example with customized graphical parameters
plot(
  bpca(gabriel1971,
    d = 1:3,
    method = "jk"
  ),
  main = "gabriel1971 - jk",
  sub = "The graphical parameters are working fine!",
  var.pch = "+",
  var.cex = .6,
  var.color = rainbow(9),
  obj.pch = "*",
  obj.cex = .8,
  obj.col = rainbow(8),
  ref.lty = "solid",
  ref.col = "red",
  angle = 70
)

##
## Alternative 3D reduction using a different starting dimension
##

bp <- bpca(gabriel1971,
  d = 2:4
)

plot(bp)

# Explore the object created by bpca()
class(bp)
names(bp)
str(bp)

summary(bp)
bp$call
bp$eigenvalues
bp$eigenvectors
bp$number
bp$importance
bp$coord
bp$var.rb
bp$var.rd

# Changing the angle between x (PC2) and y (PC3) axis
plot(bp,
  angle = 65
)

devAskNewPage(oask)

##
## dt.tools demo:
## vector lengths, pairwise angles, and projected correlations
## for numeric variables in a data.frame or matrix (n x p)
##

dt <- dt.tools(iris, 2) # Non-numeric columns are ignored internally.

# Explore the object created by dt.tools()
class(dt)
names(dt)
str(dt)

dt$length
dt$angle
dt$r
dt

# Check consistency with base cor()
(iris.tools <- round(
  dt.tools(iris,
    center = 2
  )$r,
  5
))

(iris.obsv <- round(
  cor(iris[-5]),
  5
))

all(iris.tools == iris.obsv)

##
## Main bpca demo script
## Covers 2D/3D workflows, diagnostics, and plotting modes.
##

oask <- devAskNewPage(dev.interactive(orNone = TRUE))

inspect_bpca <- function(bp) {
  cat("==========================================\n")
  cat("       BPCA OBJECT INSPECTION\n")
  cat("==========================================\n\n")

  # Basic Info
  cat("Class:", class(bp), "\n")
  cat(
    "Available components:",
    paste(names(bp),
      collapse = ", "
    ), "\n\n"
  )

  # Summary
  cat("--- Summary ---\n")
  print(summary(bp))
  cat("\n")

  # Components and Importance
  cat(
    "Number of components:",
    bp$number,
    "\n"
  )
  cat("Importance of components (Rounded):\n")
  print(round(
    bp$importance,
    3
  ))
  cat("\n")

  # Coordinates (Top 3)
  cat("--- Object Coordinates (First 3) ---\n")
  n_obj <- min(
    3,
    nrow(bp$coord$objects)
  )
  print(bp$coord$objects[seq_len(n_obj),
    drop = FALSE
  ])
  cat("\n")

  cat("--- Variable Coordinates (First 3) ---\n")
  n_var <- min(
    3,
    nrow(bp$coord$variables)
  )
  print(bp$coord$variables[seq_len(n_var),
    drop = FALSE
  ])
  cat("\n==========================================\n")
}


## ------------------------------------------------------------------
## 1) Basic 2D workflow
## ------------------------------------------------------------------
bp2 <- bpca(gabriel1971)
plot(bp2,
  main = "gabriel1971 - 2D"
)
inspect_bpca(bp2)

## Customize aesthetics in 2D
plot(
  bpca(gabriel1971,
    method = "sqrt"
  ),
  main = "gabriel1971 - sqrt",
  sub = "Customized aesthetics",
  var.cex = .6,
  var.color = rainbow(ncol(gabriel1971)),
  var.pch = "v",
  obj.pch = "o",
  obj.cex = .5,
  obj.col = rainbow(nrow(gabriel1971)),
  obj.pos = 1,
  obj.offset = .5
)

## ------------------------------------------------------------------
## 2) Basic 3D workflow (static)
## ------------------------------------------------------------------
bp3 <- bpca(gabriel1971,
  d = 1:3
)
plot(bp3,
  main = "gabriel1971 - 3D static"
)
inspect_bpca(bp3)

## Customize aesthetics in 3D static
plot(
  bpca(gabriel1971,
    d = 1:3,
    method = "jk"
  ),
  main = "gabriel1971 - jk",
  sub = "Customized aesthetics",
  var.pch = "+",
  var.cex = .6,
  var.color = rainbow(ncol(gabriel1971)),
  obj.pch = "*",
  obj.cex = .8,
  obj.col = rainbow(nrow(gabriel1971)),
  ref.lty = "solid",
  ref.col = "red",
  angle = 70
)

## Alternative 3D projection range
bp3_alt <- bpca(gabriel1971,
  d = 2:4
)
plot(bp3_alt,
  main = "gabriel1971 - 3D (PC2 to PC4)"
)
inspect_bpca(bp3_alt)

## ------------------------------------------------------------------
## 3) Interactive identification
## ------------------------------------------------------------------
if (dev.interactive()) {
  plot(bp2,
    obj.names = FALSE,
    obj.identify = TRUE,
    main = "2D object identification"
  )

  plot(bp2,
    obj.names = FALSE,
    obj.labels = c(
      "toi",
      "kit",
      "bat",
      "ele",
      "wat",
      "rad",
      "tv",
      "ref"
    ),
    obj.identify = TRUE,
    main = "2D object identification (custom labels)"
  )

  plot(bp3,
    obj.names = FALSE,
    obj.identify = TRUE,
    main = "3D object identification"
  )

  plot(bp3,
    obj.names = FALSE,
    obj.labels = c(
      "toi",
      "kit",
      "bat",
      "ele",
      "wat",
      "rad",
      "tv",
      "ref"
    ),
    obj.identify = TRUE,
    main = "3D object identification (custom labels)"
  )
}

## ------------------------------------------------------------------
## 4) Data tools helper
## ------------------------------------------------------------------
dt <- dt.tools(iris,
  center = 2
)
class(dt)
names(dt)
str(dt)
dt$length
dt$angle
dt$r

(iris.tools <- round(dt$r, 5))
(iris.obsv <- round(cor(iris[-5]), 5))
all(iris.tools == iris.obsv)

## ------------------------------------------------------------------
## 5) Grouping example (iris)
## ------------------------------------------------------------------
plot(bpca(iris[-5]),
  var.cex = .7,
  obj.names = FALSE,
  obj.cex = 1.5,
  obj.col = c(
    "red",
    "green3",
    "blue"
  )[as.numeric(iris$Species)],
  obj.pch = c(
    "+",
    "*",
    "-"
  )[as.numeric(iris$Species)],
  main = "iris - 2D grouping"
)

plot(
  bpca(iris[-5],
    d = 1:3
  ),
  var.color = c("blue", "red"),
  var.cex = 1,
  obj.names = FALSE,
  obj.cex = 1,
  obj.col = c(
    "red",
    "green3",
    "blue"
  )[as.numeric(iris$Species)],
  obj.pch = c(
    "+",
    "*",
    "-"
  )[as.numeric(iris$Species)],
  main = "iris - 3D grouping"
)

## ------------------------------------------------------------------
## 6) Quality diagnostics with var.rb and qbpca
## ------------------------------------------------------------------
# Enable pause between plots
par(ask = TRUE)

for (m in c("sqrt", "jk", "gh", "hj")) {
  bp_m <- bpca(gabriel1971,
    method = m,
    var.rb = TRUE
  )
  q_m <- qbpca(gabriel1971, bp_m)

  plot(q_m,
    highlight.width = 0.25,
    main = paste0("Method: ", m)
  )
}


for (m in c("sqrt", "jk", "gh", "hj")) {
  bp_m3 <- bpca(gabriel1971,
    method = m,
    d = 1:3,
    var.rb = TRUE
  )
  q_m3 <- qbpca(
    gabriel1971,
    bp_m3
  )
  plot(q_m3,
    highlight.width = 0.25,
    main = paste0(m, " - 3D")
  )
}

# Disable pause
par(ask = FALSE)

## var.rd highlights poor projected correlations
bp_rd <- bpca(gabriel1971,
  method = "hj",
  var.rb = TRUE,
  var.rd = TRUE,
  limit = 3
)
bp_rd$var.rd
plot(
  bpca(gabriel1971,
    method = "hj",
    d = 3:4
  ),
  main = "HJ (PC3-PC4)"
)

## ------------------------------------------------------------------
## 7) Plot modes with ontario data
## ------------------------------------------------------------------
data(ontario)
obj.lab <- paste("g",
  1:18,
  sep = ""
)

plot(bpca(ontario),
  main = "Ontario - basic"
)

plot(bpca(ontario),
  obj.labels = obj.lab,
  main = "Ontario - custom labels"
)

plot(bpca(ontario),
  type = "eo",
  obj.cex = 1,
  main = "type='eo' (default object)"
)

plot(bpca(ontario),
  type = "eo",
  obj.id = 7,
  obj.cex = 1,
  main = "type='eo' (object 7)"
)

plot(bpca(ontario),
  type = "eo",
  obj.labels = obj.lab,
  obj.id = "g7",
  obj.cex = 1,
  main = "type='eo' (object g7)"
)

plot(bpca(ontario),
  type = "ev",
  var.cex = 1,
  main = "type='ev' (default variable)"
)

plot(bpca(ontario),
  type = "ev",
  var.id = "E7",
  obj.labels = obj.lab,
  var.cex = 1,
  main = "type='ev' (variable E7)"
)

cl <- 1:3
plot(bpca(iris[-5]),
  type = "ev",
  var.id = 1,
  obj.names = FALSE,
  obj.col = cl[as.numeric(iris$Species)],
  main = "type='ev' with groups"
)

legend("topleft",
  legend = levels(iris$Species),
  text.col = cl,
  pch = 19,
  col = cl,
  cex = .9,
  box.lty = 0
)

plot(bpca(ontario),
  type = "co",
  main = "type='co' (default objects)"
)

plot(bpca(ontario),
  type = "co",
  obj.labels = obj.lab,
  main = "type='co' with labels"
)

plot(bpca(ontario),
  type = "co",
  obj.labels = obj.lab,
  obj.id = 13:14,
  main = "type='co' (objects 13 and 14)"
)

plot(bpca(ontario),
  type = "co",
  obj.labels = obj.lab,
  obj.id = c("g7", "g13"),
  main = "type='co' (objects g7 and g13)"
)

plot(bpca(ontario),
  type = "cv",
  main = "type='cv'"
)

plot(bpca(ontario),
  type = "ww",
  main = "type='ww'"
)

plot(bpca(ontario),
  type = "dv",
  main = "type='dv'"
)

plot(bpca(ontario),
  type = "ms",
  main = "type='ms'"
)

plot(bpca(ontario),
  type = "ro",
  main = "type='ro'"
)

plot(bpca(ontario),
  type = "rv",
  main = "type='rv'"
)

## Out-of-range examples for defensive behavior
plot(bpca(iris[-5]),
  type = "eo",
  obj.id = 42,
  obj.cex = 1
)

plot(bpca(iris[-5]),
  type = "ev",
  var.id = "Sepal.Width"
)

plot(bpca(iris[-5]),
  type = "ev",
  var.id = "Sepal.Width",
  var.factor = .3
)

devAskNewPage(oask)

##
## 3D quality comparison using var.rb + qbpca
##

oask <- devAskNewPage(dev.interactive(orNone = TRUE))

## Differences between methods of factorization
# SQRT
bp_sqrt <- bpca(gabriel1971,
  method = "sqrt",
  d = 1:3,
  var.rb = TRUE
)

qbp_sqrt <- qbpca(
  gabriel1971,
  bp_sqrt
)

plot(qbp_sqrt,
  main = "sqrt - 3D \n (poor)"
)

# JK
bp_jk <- bpca(gabriel1971,
  method = "jk",
  d = 1:3,
  var.rb = TRUE
)

qbp_jk <- qbpca(
  gabriel1971,
  bp_jk
)

plot(qbp_jk,
  main = "jk - 3D \n (very poor)"
)

# HJ
bp_hj <- bpca(gabriel1971,
  method = "hj",
  d = 1:3,
  var.rb = TRUE
)

qbp_hj <- qbpca(
  gabriel1971,
  bp_hj
)

plot(qbp_hj,
  main = "hj - 3D \n (wow!)"
)

# GH
bp_gh <- bpca(gabriel1971,
  method = "gh",
  d = 1:3,
  var.rb = TRUE
)

qbp_gh <- qbpca(
  gabriel1971,
  bp_gh
)

plot(qbp_gh,
  main = "gh - 3D \n (wow!)"
)

devAskNewPage(oask)

##
## Static 2D bpca workflow with base graphics
##

oask <- devAskNewPage(dev.interactive(orNone = TRUE))

bp <- bpca(gabriel1971)

plot(bp)

# Explore the object created by bpca()
class(bp)
names(bp)
str(bp)

summary(bp)
bp$call
bp$eigenvalues
bp$eigenvectors
bp$number
bp$importance
bp$coord
bp$coord$objects
bp$coord$variables
bp$var.rb
bp$var.rd

# Example with customized graphical parameters
plot(
  bpca(gabriel1971,
    method = "sqrt"
  ),
  main = "gabriel1971 - sqrt",
  sub = "The graphical parameters are working fine!",
  var.cex = .6,
  var.color = rainbow(9),
  var.pch = "v",
  obj.pch = "o",
  obj.cex = .5,
  obj.col = rainbow(8),
  obj.pos = 1,
  obj.offset = .5
)

devAskNewPage(oask)

# José Cláudio Faria
var.rdf <- function(x,
                    var.rb,
                    limit)
{
  dif <- 100 * abs(var.rb - cor(x))

  big <- dif > limit
  leq <- dif <= limit

  dif[big] <- '*'
  dif[leq] <- ''
  diag(dif) <- '-'

  var.rd <- dif
  dimnames(var.rd) <- list(dimnames(x)[[2]],
                           dimnames(x)[[2]])

  return(as.data.frame(var.rd))
}

##
## Comparative example based on Yan and Kang (GGE biplot analysis)
##

oask <- devAskNewPage(dev.interactive(orNone = TRUE))

bp <- bpca(t(gge2003),
  var.rb = TRUE
)

as.dist(bp$var.rb)

op <- par(no.readonly = TRUE)

par(mfrow = c(1, 2))

plot(bpca(gge2003),
  main = "Columns as variables",
  var.color = 1,
  obj.col = 2:4,
  obj.cex = .8
)

plot(bpca(t(gge2003)),
  main = "Rows as variables",
  var.color = 1,
  obj.col = c(2:4, 2),
  obj.cex = .8
)

par(op)

devAskNewPage(oask)

##
## Example of `var.rd=TRUE` as a biplot quality measure (2D)
## Mainly recommended for large datasets.
##

oask <- devAskNewPage(dev.interactive(orNone = TRUE))

bp_hj <- bpca(gabriel1971,
  method = "hj",
  var.rb = TRUE,
  var.rd = TRUE,
  limit = 4
)

bp_hj$var.rd

# RUR followed by CRISTIAN contains information in dimensions that
# weren't contemplated by the biplot reduction (PC3).
# Among all variables, RUR followed by CRISTIAN is poorly represented in 2D.
# biplot.

# Basic quality plot (default highlight threshold = 10)
plot(
  qbpca(
    gabriel1971,
    bp_hj
  ),
  highlight.width = 0.25,
  main = "Observed vs projected correlations"
)

# Details
qbp_2d <- qbpca(
  gabriel1971,
  bp_hj
)

# New plot.qbpca features:
# - limit: threshold for poor representation (in % difference)
# - highlight.col / highlight.lty / highlight.pad / highlight.width:
#   style and width of dashed highlight boxes
# - pair.labels + automatic rotated labels for variable pairs
plot(qbp_2d,
  limit = 4,
  highlight.col = "gray70",
  highlight.lty = "dashed",
  highlight.pad = 0.04,
  highlight.width = 0.25,
  pair.labels = TRUE,
  label.side = "bottom",
  label.angle = 45,
  label.cex = 0.5,
  label.offset = 1 / 30,
  main = "Automatic pair labels and poor-projection highlights"
)

# Alternative view: labels on top
plot(qbp_2d,
  limit = 4,
  highlight.col = "gray55",
  highlight.width = 0.35,
  pair.labels = TRUE,
  label.side = "top",
  label.angle = 35,
  label.cex = 0.45,
  label.offset = 0.04
)

# Graphical visualization of the importance of the variables not contemplated
# in the prior (2D) reduction: RUR and CRISTIAN are the most important variables in PC3.
plot(
  bpca(gabriel1971,
    method = "hj",
    d = 3:4
  ),
  main = "hj"
)

devAskNewPage(oask)

##
## Interactive 3D bpca workflow with rgl
##

# rgl::open3d()
plot(
  pca <- bpca(iris[-5],
    d = 1:3
  ),
  rgl.use = TRUE,
  var.color = "brown",
  var.cex = 1.2,
  obj.names = FALSE,
  obj.cex = .3,
  obj.col = c(
    "blue",
    "green",
    "red"
  )[as.numeric(iris$Species)],
  simple.axes = TRUE
)

scores <- pca$coord$objects

ell <- rgl::ellipse3d(cov(scores),
  center = colMeans(scores),
  level = 0.68
)

rgl::plot3d(ell,
  col = "gray",
  alpha = 0.2,
  add = TRUE
)

rgl::play3d(rgl::spin3d(axis = c(1, 2, 3)),
  duration = 12
)

# This graphic style was suggested by Michael Friendly (York University).
# Tip: interact with the graphic using the mouse.
# left button: click and drag to rotate;
# right button: click and drag to zoom.

# José Cláudio Faria
plot.bpca.2d <- function(x,
                         type=c('bp', 'eo', 'ev', 'co', 'cv', 'ww', 'dv', 'ms', 'ro', 'rv'),
                         c.color='darkgray',
                         c.lwd=1,
                         c.number=5,
                         c.radio=1,
                         obj.id=1:2,
                         var.id=1,
                         base.color='red3',
                         base.lty='dotted',
                         proj.color='gray',
                         proj.lty='dotted',
                         a.color='blue',
                         a.lty='solid',
                         a.lwd=2,
                         a.length=.1,
                         ref.lines=TRUE,
                         ref.color='navy',
                         ref.lty='dotted',
                         var.factor=NULL,
                         var.color='red3',
                         var.lty='solid',
                         var.pch=20,
                         var.pos=4,
                         var.cex=.6,
                         var.offset=.2,
                         obj.factor=1,
                         obj.color='black',
                         obj.pch=20,
                         obj.pos=4,
                         obj.cex=.6,
                         obj.offset=.2,
                         obj.names=TRUE,
                         obj.labels,
                         obj.identify=FALSE,
                         xlim, ylim, xlab, ylab, ...)
{
  draw.obj <-
    function()
    {   
      # objects
      if(obj.names) {
        points(x=coobj[,d1],
               y=coobj[,d2],
               pch=obj.pch,
               col=obj.color,
               cex=obj.cex, ...)

        text(x=coobj[,d1],
             y=coobj[,d2],
             labels=obj.labels,
             pos=obj.pos,
             offset=obj.offset,
             col=obj.color,
             cex=obj.cex, ...)
      } else {
        points(x=coobj[,d1],
               y=coobj[,d2],
               pch=obj.pch,
               col=obj.color,
               cex=obj.cex, ...)
      }
    }  

  draw.var <-
    function()
    {
      # Coordenadas escalonadas
      vx <- covar[,d1] * var.factor
      vy <- covar[,d2] * var.factor

      # Desenha os pontos (pontas dos vetores)
      points(x=vx,
             y=vy,
             pch=var.pch,
             col=var.color,
             cex=var.cex, ...)

      # Desenha os textos com ajuste radial e folga (offset)
      for (i in 1:nrow(covar)) {
        # 1. Determina o ajuste (0 = esquerda, 1 = direita)
        adj.x <- ifelse(vx[i] >= 0, 0, 1)

        # 2. Cria a folga baseada no var.offset e no sinal da coordenada
        # Dividimos por um fator (ex: 10 ou 20) para a folga ser proporcional
        off.x <- sign(vx[i]) * (var.offset * 0.2)
        off.y <- sign(vy[i]) * (var.offset * 0.2)

        text(x=vx[i] + off.x,
             y=vy[i] + off.y,
             labels=rownames(covar)[i],
             adj=c(adj.x, 0.5), # 0.5 centraliza o texto verticalmente na ponta
             col=var.color,
             cex=var.cex, ...)
      }
    }

  draw.var.seg <-
    function()
    {   
      # var segments
      segments(x0=0,
               y0=0,
               x1=covar[,d1] * var.factor,
               y1=covar[,d2] * var.factor,
               col=var.color,
               lty=var.lty, ...)
    }

  draw.circles <-
    function()
    {  
      # concentric circles (0,0)
      for (i in 1:c.number)
        symbols(x=0,
                y=0,
                circles=c.radio * i * var.factor,
                add=TRUE,
                inches=FALSE,
                fg=c.color,
                lwd=c.lwd, ...)
    }

  if (!inherits(x, 'bpca.2d'))
    stop("Use this function only with 'bpca.2d' class!")

  coobj <- x$coord$objects
  covar <- x$coord$variables
  d1 <- x$number[1]
  d2 <- x$number[2]

  if (is.null(var.factor))
    var.factor <- max(abs(coobj)) / max(abs(covar))

  scores <- rbind(coobj,
                  covar * var.factor)

  if (missing(obj.labels))
    obj.labels <- rownames(coobj)

#  if (missing(xlim) || missing(ylim)) {
#    ms <- max(abs(scores)) * 1.1
#    msp <- c(-ms, ms)
#  }
#
#  if (missing(xlim))
#    xlim <- msp
#  if (missing(ylim))
#    ylim <- msp

  # Calculo automatico de xlim e ylim se nao fornecidos
  if (missing(xlim) || missing(ylim)) {
    # Extrai ranges atuais
    rx <- range(scores[, d1], na.rm=TRUE)
    ry <- range(scores[, d2], na.rm=TRUE)

    # Buffer proporcional ao cex e ao tamanho do grafico (ajustavel)
    # 15% de folga costuma acomodar bem os textos laterais
    buffer_x <- diff(rx) * (max(var.cex, obj.cex) * 0.20)
    buffer_y <- diff(ry) * (max(var.cex, obj.cex) * 0.20)

    if (missing(xlim))
      xlim <- c(rx[1] - buffer_x, rx[2] + buffer_x)

    if (missing(ylim))
      ylim <- c(ry[1] - buffer_y, ry[2] + buffer_y)
  }

  if (missing(xlab) || missing(ylab)) {
    eigv <- x$eigenvalues
    prop <- 100 * eigv^2 / sum(eigv^2)
    labs <- paste('PC',
                  d1:d2,
                  ' (',
                  round(prop[d1:d2], 2),
                  '%)', 
                  sep='')
  }

  if (missing(xlab))
    xlab <- labs[1]
  if (missing(ylab))
    ylab <- labs[2]

  plot(scores,
       xlim=xlim,
       ylim=ylim,
       xlab=xlab,
       ylab=ylab,
       type='n', ...)

  if (ref.lines)
    abline(h=0,
           v=0,
           col=ref.color,
           lty=ref.lty, ...)

  switch(match.arg(type), 
         bp={ # basic biplot 2d
           draw.obj()
           draw.var()
           draw.var.seg()

           # identification of objects with mouse
           if(obj.identify)
             identify(x=coobj,
                      labels=obj.labels,
                      cex=obj.cex)
         }, 
         eo={ # evaluate an object
           if (any(class(obj.id) == c('numeric', 'integer'))) 
             obj.lab <- obj.labels[obj.id[1]]
           else {
             if (obj.id %in% obj.labels){
               obj.lab <- obj.labels[match(obj.id, 
                                           obj.labels)] 
               obj.id <- match(obj.id, 
                               obj.labels)
             }
             else
               stop("'obj.id' do not match with 'obj.labels'!")
           }

           draw.var()

           abline(a=0,
                  b=coobj[obj.id,d2] / coobj[obj.id,d1],
                  col=base.color,
                  lty=base.lty, ...)

           abline(a=0,
                  b=-coobj[obj.id,d1] / coobj[obj.id,d2],
                  col=base.color,
                  lty=base.lty, ...)

           arrows(x0=0,
                  y0=0,
                  x1=coobj[obj.id[1],d1] * obj.factor,
                  y1=coobj[obj.id[1],d2] * obj.factor,
                  col=a.color,
                  lty=a.lty,
                  lwd=a.lwd,
                  length=a.length, ...)

           points(x=coobj[obj.id[1],d1] * obj.factor,
                  y=coobj[obj.id[1],d2] * obj.factor,
                  pch=obj.pch,
                  col=obj.color,
                  cex=obj.cex, ...)


           text(x=coobj[obj.id[1],d1] * obj.factor, 
                y=coobj[obj.id[1],d2] * obj.factor,
                labels=obj.lab,
                pos=obj.pos,
                offset=obj.offset,
                col=obj.color,
                cex=obj.cex, ...)

           x <- solve(cbind(c(-coobj[obj.id[1],d2],
                              coobj[obj.id[1],d1]),
                            c(coobj[obj.id[1],d1],
                              coobj[obj.id[1],d2])),
                      rbind(0,
                            as.numeric(covar[,c(d1, d2)] %*%
                                       coobj[obj.id[1],c(d1, d2)])))
           segments(x0=covar[,d1],
                    y0=covar[,d2],
                    x1=x[1,],
                    y1=x[2,],
                    lty=proj.lty,
                    col=proj.color)
         }, 
         ev={ # evaluate a variable
           draw.obj()

           abline(a=0,
                  b=covar[var.id,d2] / covar[var.id,d1],
                  col=base.color,
                  lty=base.lty, ...)

           abline(a=0,
                  b=-covar[var.id,d1] / covar[var.id,d2],
                  col=base.color,
                  lty=base.lty, ...)

           arrows(x0=0,
                  y0=0,
                  x1=covar[var.id,d1] * var.factor,
                  y1=covar[var.id,d2] * var.factor,
                  col=a.color,
                  lty=a.lty,
                  lwd=a.lwd,
                  length=a.length, ...)

           points(x=covar[var.id,d1] * var.factor, 
                  y=covar[var.id,d2] * var.factor, 
                  pch=var.pch,
                  col=var.color,
                  cex=var.cex, ...)

           text(x=covar[var.id,d1] * var.factor,
                y=covar[var.id,d2] * var.factor,
                labels=ifelse(mode(var.id) == 'numeric', rownames(covar)[var.id], var.id), 
                pos=var.pos,
                offset=var.offset,
                col=var.color,
                cex=var.cex, ...)

           x <- solve(cbind(c(-covar[var.id,d2],
                              covar[var.id,d1]),
                            c(covar[var.id,d1],
                              covar[var.id,d2])),
                      rbind(0,
                            as.numeric(coobj[,c(d1, d2)] %*%
                                       covar[var.id,c(d1, d2)])))

           segments(x0=coobj[,d1],
                    y0=coobj[,d2],
                    x1=x[1,],
                    y1=x[2,],
                    lty=proj.lty,
                    col=proj.color, ...)
         }, 
         co={ # compare two objects
           if (any(class(obj.id) == c('character', 'factor'))) 
             if (obj.id[1] %in% obj.labels &
                 obj.id[2] %in% obj.labels) {
               obj.id[1] <- match(obj.id[1], obj.labels)
               obj.id[2] <- match(obj.id[2], obj.labels)
             } else 
               stop("At last one 'obj.id' do not match with 'obj.labels'!")

         draw.obj()
         draw.var()

         symbols(x=coobj[obj.id[1],d1],
                 y=coobj[obj.id[1],d2],
                 circles=c.radio,
                 add=TRUE,
                 inches=FALSE,
                 fg=c.color, 
                 lwd=c.lwd, ...)

         symbols(x=coobj[obj.id[2],d1],
                 y=coobj[obj.id[2],d2],
                 circles=c.radio,
                 add=TRUE,
                 inches=FALSE,
                 fg=c.color, 
                 lwd=c.lwd, ...)

         segments(x0=coobj[obj.id[1],d1],
                  y0=coobj[obj.id[1],d2],
                  x1=coobj[obj.id[2],d1],
                  y1=coobj[obj.id[2],d2],
                  col=proj.color,
                  lty=proj.lty, ...)

         abline(a=0,
                b=-(coobj[obj.id[1],d1] - 
                    coobj[obj.id[2],d1]) / 
                (coobj[obj.id[1],d2] - 
                 coobj[obj.id[2],d2]),
                col=base.color,
                lty=base.lty, ...)
         }, 
         cv={ # compare two variables
           draw.obj()
           draw.var()
           draw.var.seg()
           draw.circles()
         }, 
         ww={ ## which won where/what (from 1.0-1)
           draw.obj()
           draw.var()

           indice <- c(chull(coobj[,d1],
                             coobj[,d2]))

           polygon(x=coobj[indice,d1],
                   y=coobj[indice,d2],
                   border=proj.color,
                   lty=proj.lty, ...)

           i <- 1
           while (is.na(indice[i + 1]) == FALSE) {
             m <- (coobj[indice[i],d2] -
                   coobj[indice[i + 1],d2]) / 
             (coobj[indice[i],d1] - 
              coobj[indice[i + 1],d1])

             mperp <- -1 / m

             c2 <- coobj[indice[i + 1],d2] -
               m * coobj[indice[i + 1],d1]

             xint <- -c2/(m - mperp)

             xint <- ifelse(xint < 0,
                            min(covar[,d1], 
                                coobj[,d1]),
                            max(covar[,d1], 
                                coobj[,d1]))

             yint <- mperp * xint

             segments(x0=0,
                      y0=0,
                      x1=xint,
                      y1=yint,
                      col=base.color,
                      lty=base.lty, ...)

             i <- i + 1
           }

           m <- (coobj[indice[i],d2] -
                 coobj[indice[1],d2]) /
           (coobj[indice[i],d1] -
            coobj[indice[1],d1])

           mperp <- -1 / m

           c2 <- coobj[indice[i],d2] - 
             m * coobj[indice[i],d1]

           xint <- -c2 / (m - mperp)

           xint <- ifelse(xint < 0,
                          min(covar[,d1], 
                              coobj[,d1]),
                          max(covar[,d1], 
                              coobj[,d1]))

           yint <- mperp * xint

           segments(x0=0,
                    y0=0,
                    x1=xint,
                    y1=yint,
                    col=base.color,
                    lty=base.lty, ...)
         }, 
         dv={ # discrimitiveness vs. representativeness
           draw.obj()
           draw.var()
           draw.circles()
           draw.var.seg()

           arrows(x0=0,
                  y0=0,
                  x1=mean(covar[, d1] * var.factor),
                  y1=mean(covar[, d2] * var.factor), # 
                  col=a.color,
                  lty=a.lty,
                  lwd=a.lwd,
                  length=a.length, ...)

           points(mean(covar[, d1] * var.factor),
                  mean(covar[, d2] * var.factor),
                  pch=1,
                  cex=3,
                  col='blue', ...)

           abline(a=0,
                  b=mean(covar[,d2]) / mean(covar[,d1]),
                  col=var.color,
                  lty=base.lty, ...)                                                 
         }, 
         ms={ # means vs. stability
           m1 <- mean(covar[,d1] * var.factor)
           m2 <- mean(covar[,d2] * var.factor)        

           abline(a=0,
                  b=m2 / m1,
                  col=base.color,
                  lty=base.lty, ...)

           abline(a=0,
                  b=-m1/m2,
                  col=base.color,
                  lty=base.lty, ...)

           arrows(x0=0,
                  y0=0,
                  x1=m1,
                  y1=m2,
                  col=a.color,
                  lty=a.lty,
                  lwd=a.lwd,
                  length=a.length, ...)

           draw.obj()
           draw.var()

           for (i in 1:c.number)
             symbols(x=m1,
                     y=m2,
                     circles=c.radio * i * var.factor,
                     add=TRUE,
                     inches=FALSE,
                     fg=c.color,
                     lwd=c.lwd, ...)

           for (i in 1:nrow(coobj))
           {
             x <- solve(matrix(c(-m2, 
                                 m1, 
                                 m1, 
                                 m2),
                               nrow=2),
                        matrix(c(0, 
                                 m2 * coobj[i,d2] +
                                   m1 * coobj[i,d1]),
                               ncol=1))

             segments(x0=coobj[i,d1],
                      y0=coobj[i,d2],
                      x1=x[1],
                      y1=x[2],
                      col=proj.color,
                      lty=proj.lty, ...)
           }                        
         }, 
         ro={ # rank objects with ref. to the ideal variable
           m1 <- mean(covar[,d1])
           m2 <- mean(covar[,d2])

           abline(a=0,
                  b=m2 / m1,
                  col=base.color,
                  lty=base.lty, ...)

           abline(a=0,
                  b=-m1 / m2,
                  col=base.color,
                  lty=base.lty, ...)

           draw.obj()
           draw.var()

           cox <- 0
           coy <- 0

           for (i in 1:nrow(coobj))
           {
             x <- solve(matrix(c(-m2, 
                                 m1, 
                                 m1, 
                                 m2),
                               nrow=2),
                        matrix(c(0, 
                                 m2 * coobj[i,d2] + m1 * coobj[i,d1]),
                               ncol=1))
             if (sign(x[1]) == sign(m1))
               if(abs(x[1]) > abs(cox))
               {
                 cox <- x[1]
                 coy <- x[2]
               }
           }

           arrows(x0=0,
                  y0=0,
                  x1=cox,
                  y1=coy,
                  col=a.color,
                  lty=a.lty,
                  lwd=a.lwd,
                  length=a.length, ...)

           for (i in 1:c.number)
             symbols(x=cox,
                     y=coy,
                     circles=c.radio * i,
                     add=TRUE,
                     inches=FALSE,
                     fg=c.color, 
                     lwd=c.lwd, ...)
         }, 
         rv={ # rank variables with ref. to the ideal object
           draw.obj()
           draw.var()

           m1 <- mean(covar[,d1])
           m2 <- mean(covar[,d2])

           abline(a=0,
                  b=m2 / m1,
                  col=var.color,
                  lty="solid", ...)

           abline(a=0,
                  b=-m1 / m2,
                  col=var.color,
                  lty="solid", ...)

           symbols(x=m1,
                   y=m2,
                   circles=0.1,
                   add=TRUE,
                   inches=FALSE,
                   fg=c.color, 
                   lwd=c.lwd, ...)

           mod <- max((covar[,d1]^2 + covar[,d2]^2)^0.5)
           cox <- sign(m1) * (mod^2 / (1 + m2^2 / m1^2))^0.5
           coy <- (m2 / m1) * cox

           arrows(x0=0,
                  y0=0,
                  x1=cox,
                  y1=coy,
                  col=a.color,
                  lty=a.lty,
                  lwd=a.lwd,
                  length=a.length, ...)

           for (i in 1:c.number)
             symbols(x=cox,
                     y=coy,
                     circles=c.radio*i,
                     add=TRUE,
                     inches=FALSE,
                     fg=c.color, 
                     lwd=c.lwd, ...)
         })
}  

# José Cláudio Faria
plot.qbpca <- function(x,
                       xlab='Index',
                       ylab='r',
                       pch=c(1,8),
                       col=c(4,2), ...)
{
  if (!inherits(x, 'qbpca'))
    stop("Use this function only with 'qbpca' class!")

  plot(x$obs,
       ylim=c(-1,1),
       xlab=xlab,
       ylab=ylab,
       pch=pch[1],
       col=col[1], ...)

  points(x$var.rb,
         col=col[2],
         pch=pch[2])

  legend('bottomleft',
         c('r', 'r.rb'),
         pch=pch,
         col=col,
         title='legend')
}

# José Cláudio Faria
qbpca <- function(x,
                  bpca)
{
  if (missing(x) || missing(bpca))
    stop('Please, check the parameters x and bpca!')

  if (length(bpca$var.rb) == 1)
    if (is.na(bpca$var.rb))
      stop("Please, check parameter 'bpca': var.rb is not available (NA)!")

  qb <- data.frame(obs=cor(x)[lower.tri(cor(x))],
                   var.rb=bpca$var.rb[lower.tri(bpca$var.rb)])

  class(qb) <- c('qbpca', 'data.frame')
  invisible(qb)
}

# José Cláudio Faria
print.xtable.bpca <- function(x,
                              hline.after=getOption("xtable.hline.after", NULL),
                              include.colnames=getOption("xtable.include.colnames", FALSE),
                              add.to.row=getOption("xtable.add.to.row", NULL), 
                              sanitize.text.function=getOption("xtable.sanitize.text.function", NULL), 
                              sanitize.rownames.function=getOption("xtable.sanitize.rownames.function", sanitize.text.function),
                              sanitize.colnames.function=getOption("xtable.sanitize.rownames.function", sanitize.text.function), ...)
{
  aux_attr <- attr(x,'align')
  attr(x,'align') <- c('l', aux_attr)

  if(is.null(sanitize.rownames.function)){
    morerow <- function(x) paste("&",
                                 x,
                                 collpase='')
    sanitizerownamesfunction <- morerow
  }else{
    morerow <- function(x) paste("&",
                                 sanitize.rownames.function(x),
                                 collpase='')
    sanitizerownamesfunction <- morerow
  }

  if(is.null(sanitize.colnames.function)){
    sanitize.colnames.function <- function(x) x
  }

  if(is.null(add.to.row)){
    variables <- rownames(x)[1:(length(rownames(x))-3)]
    nvariables <- length(variables)
    components <- dimnames(x)[[2]]
    ncomponents <- length(components)
    whatcomponents <- as.numeric(gsub("[A-Za-z]*","",components))

    label_eigenvec <- unique(gsub("(\\\\_[\\s\\S]*)",
                                  "",
                                  variables,
                                  perl=TRUE))
    label_eigenval <- rownames(x)[length(rownames(x))-2]
    label_variance <- rownames(x)[-(1:(nvariables+1))]

    newvariables <- gsub(paste(label_eigenvec,
                               "\\\\_",
                               sep=""),
                         "",
                         variables)

    head1 <- paste("&&\\multicolumn{",
                   ncomponents,
                   "}{c}{",
                   sanitize.colnames.function(label_eigenval),
                   "} \\\\ \\cline{3-",
                   length(aux_attr)+1,
                   "}\n",
                   sep="")

    aux_head21 <- c("&& ",
                    rep("",
                        ncomponents-1))

    aux_head22 <- paste(components,
                        " $(\\lambda_",
                        whatcomponents,
                        "=",
                        round(as.numeric(x[nvariables+1,]),
                              attr(x,
                                   'digits')[2]),
                        ")$",
                        sep='')         

    aux_head23 <- paste(aux_head21,
                        sanitize.colnames.function(aux_head22),
                        collapse='&') 

    head2 <- paste(aux_head23,
                   "\\\\ \n ",
                   collapse="")

    # A função sanitize.rownames.function deve ser aplicada ao objeto newvariables também!
    #     aux_com1 <- paste(paste("\\hline \n \\multirow{",
    #                             nvariables,
    #                             "}{*}{",
    #                             sanitize.rownames.function(label_eigenvec),
    #                             "}",
    #                             sep=''),
    #                       newvariables[1],
    #                       sep='&')
    label_eigenvec <- ifelse(is.null(sanitize.rownames.function),
                             label_eigenvec,
                             label_eigenvec <- sanitize.rownames.function(label_eigenvec))
    firstvariablerow <-ifelse(is.null(sanitize.rownames.function),
                              newvariables[1],
                              firstvariablerow <- sanitize.rownames.function(newvariables[1]))  
    aux_com1 <- paste(paste("\\hline \n \\multirow{",
                            nvariables,
                            "}{*}{",
                            label_eigenvec,
                            "}",
                            sep=''),
                      firstvariablerow,
                      sep='&') 
    
    aux_com11 <- gsub("(&\\s)",
                      "",
                      aux_com1,
                      perl=TRUE)
    aux_com2 <- paste(round(x[1,],
                            attr(x,
                                 'digits')[2]),
                      collapse='&')

    if(include.colnames){
      add.to.row <- list(pos=list(0, 0, 0, 0), 
                         command=NULL)
      aux_head01 <- paste("&",
                          colnames(x))
      aux_head02 <- paste(aux_head01, 
                          collapse="")
      head0 <- paste("&",
                     aux_head02,
                     "\\\\ \n")

      command <- c(head0,
                   head1,
                   head2,
                   paste(paste(aux_com11,
                               aux_com2,
                               sep='&'),
                         '\\\\ \n')) 
    } else {
      add.to.row <- list(pos=list(0, 0, 0), 
                         command=NULL)
      command <- c(head1,
                   head2,
                   paste(paste(aux_com11,
                               aux_com2,
                               sep='&'),
                         '\\\\ \n'))
    }

    add.to.row$command <- command
  }

  rownames(x) <- c(newvariables, 
                   label_eigenval, 
                   label_variance)

  if(is.null(hline.after)){

    hline.after <- c(-1,
                     nrow(x[-c(1,nvariables+1),])-2,
                     nrow(x[-c(1,nvariables+1),]))

  }

  print.xtable(x[-c(1,nvariables+1),],
               hline.after=hline.after,
               include.colnames=FALSE,
               sanitize.rownames.function=sanitizerownamesfunction,
               add.to.row=add.to.row,
               ...)

}

# José Cláudio Faria
summary.bpca <- function(object,
                         presentation=FALSE, ...)
{
  if (!inherits(object, 'bpca'))
    stop("Use this function only with 'bpca' class!")

  if(!presentation){

    d <- length(object$number)

    x <- list('Eigenvalue(s)' = object$eigenvalues,
              'Considered on reduction' = object$eigenvalues[object$number[1]:object$number[d]],
              'Variance retained by each' = object$eigenvalues[object$number[1]:object$number[d]]^2 / 
              sum(object$eigenvalues^2),
              'Cumulative variance retained' = cumsum(object$eigenvalues[object$number[1]:object$number[d]]^2) /
              sum(object$eigenvalues^2),
              'Prop. of total variance retained' = object$importance[1]) 

    if(object$importance[1] != object$importance[2]) 
      x$'Prop. of partial variance retained' = object$importance[2]

    class(x) <- c('summary.bpca', 'listof')

    x

  } else {

    d <- length(object$number)

    cat(' Eigenvalue(s):\t\t\t\t',
        object$eigenvalues)

    cat('\n  - Considered on reduction:\t\t',
        object$eigenvalues[object$number[1]:object$number[d]])

    cat('\n  - Variance retained by each:\t\t',
        object$eigenvalues[object$number[1]:object$number[d]]^2 /
        sum(object$eigenvalues^2))

    cat('\n  - Cumulative variance retained:\t',
        cumsum(object$eigenvalues[object$number[1]:object$number[d]]^2) /
        sum(object$eigenvalues^2))

    cat('\n  - Prop. of total variance retained:\t',
        object$importance[1])

    if(object$importance[1] != object$importance[2])
      cat('\n  - Prop. of partial variance retained:\t',
          object$importance[2])

    cat('\n')
  }
}

# José Cláudio Faria
plot.bpca.3d <- function(x,
                         rgl.use=FALSE,
                         ref.lines=TRUE,
                         ref.color='navy',
                         ref.lty=ifelse(rgl.use, NA, 'dotted'),
                         clear3d=ifelse(rgl.use, TRUE, NULL),
                         simple.axes=ifelse(rgl.use, TRUE, NULL),
                         aspect=ifelse(rgl.use, c(1, 1, 1), NULL),
                         var.factor=NULL,
                         var.color='red3',
                         var.lty=ifelse(rgl.use, NA, 'solid'),
                         var.pch=ifelse(rgl.use, NULL, 20),
                         var.pos=ifelse(rgl.use, 0, 4),
                         var.cex=ifelse(rgl.use, .8, .6),
                         var.offset=ifelse(rgl.use, NULL, .2),
                         obj.color='black',
                         obj.pch=ifelse(rgl.use, NULL, 20),
                         obj.pos=ifelse(rgl.use, 0, 4),
                         obj.cex=ifelse(rgl.use, .8, .6),
                         obj.offset=ifelse(rgl.use, NULL, .2),
                         obj.names=TRUE,
                         obj.labels,
                         obj.identify=FALSE,
                         box=FALSE,
                         angle=ifelse(rgl.use, NULL, 40),
                         xlim, ylim, zlim, xlab, ylab, zlab, ...)
{
  if (!inherits(x, 'bpca.3d'))
    stop("Use this function only with 'bpca.3d' class!")

  d1 <- x$number[1]
  d3 <- x$number[3]
  coobj <- x$coord$objects[,d1:d3]
  covar <- x$coord$variables[,d1:d3]

  if (is.null(var.factor))
    var.factor <- max(abs(coobj)) / max(abs(covar))

  scores <- rbind(coobj,
                  covar * var.factor)

  var.color <- rep_len(var.color,
                       nrow(covar))

  if (missing(obj.labels)) obj.labels <- rownames(coobj)

  if (missing(xlab) || missing(ylab) || missing(zlab)) {
    eigv <- x$eigenvalues
    prop <- 100 * eigv^2 / sum(eigv^2)
    labs <- paste0('PC',
                   d1:d3,
                   ' (',
                   round(prop[d1:d3], 2),
                   '%)')
  }

  # --- Ajuste para Limites Independentes ---
  if (missing(xlim) || missing(ylim) || missing(zlim)) {
    # Calculamos o range individual para cada PC selecionado
    rx <- range(scores[,1],
                na.rm=TRUE)
    ry <- range(scores[,2],
                na.rm=TRUE)
    rz <- range(scores[,3],
                na.rm=TRUE)

    # Aplicamos o buffer de 10% sobre a amplitude de cada eixo
    # Isso garante que a folga seja proporcional ao tamanho do eixo
    mult <- ifelse(rgl.use, 0.10, 0.05)

    buf_x <- diff(rx) * mult
    buf_y <- diff(ry) * mult
    buf_z <- diff(rz) * mult

    # Definimos os limites (incluindo o zero para as ref.lines)
    # Se quisermos as linhas de referência, o limite deve conter o zero
    if (missing(xlim))
      xlim <- c(min(rx[1] - buf_x, 0),
                max(rx[2] + buf_x, 0))

    if (missing(ylim))
      ylim <- c(min(ry[1] - buf_y, 0),
                max(ry[2] + buf_y, 0))

    if (missing(zlim))
      zlim <- c(min(rz[1] - buf_z, 0),
                max(rz[2] + buf_z, 0))
  }


  if (missing(xlab))
    xlab <- labs[1]
  if (missing(ylab))
    ylab <- labs[2]
  if (missing(zlab))
    zlab <- labs[3]

  # --- Modo scatterplot3d ---
  if(!rgl.use) {
    op <- par(no.readonly=TRUE)

    graph <- scatterplot3d::scatterplot3d(x=as.numeric(scores[,1]),
                                          y=as.numeric(scores[,2]),
                                          z=as.numeric(scores[,3]),
                                          xlim=xlim, ylim=ylim, zlim=zlim,
                                          type='n', xlab=xlab, ylab=ylab, zlab=zlab,
                                          grid=FALSE, box=box, angle=angle,
                                          y.margin.add=0.1, # Minimiza a folga do eixo Y
                                          x.ticklabs=NULL, # Força o cálculo justo
                                          ...)

    if(ref.lines) {
      graph$points3d(xlim,
                     c(0, 0),
                     c(0, 0),
                     type='l',
                     lty=ref.lty,
                     col=ref.color)

      graph$points3d(c(0, 0),
                     ylim,
                     c(0, 0),
                     type='l',
                     lty=ref.lty,
                     col=ref.color)

      graph$points3d(c(0, 0),
                     c(0, 0),
                     zlim,
                     type='l',
                     lty=ref.lty,
                     col=ref.color)
    }

    if(obj.names) {
      graph$points3d(coobj,
                     pch=obj.pch,
                     col=obj.color,
                     cex=obj.cex)

      text(graph$xyz.convert(coobj),
           labels=obj.labels,
           pos=obj.pos,
           offset=obj.offset,
           col=obj.color,
           cex=obj.cex)
    } else {
      graph$points3d(coobj,
                     pch=obj.pch,
                     col=obj.color,
                     cex=obj.cex)
    }

    for(i in 1:nrow(covar)) {
      v_curr <- covar[i,] * var.factor
      graph$points3d(rbind(c(0, 0, 0),
                     v_curr),
                     col=var.color[i],
                     type='l',
                     lty=var.lty)

      graph$points3d(matrix(v_curr, ncol=3),
                     pch=var.pch,
                     col=var.color[i],
                     cex=var.cex)

      p_v <- graph$xyz.convert(matrix(v_curr,
                                      ncol=3))

      # Ajuste radial para evitar sobreposição na projeção 2D
      off_x <- sign(v_curr[1]) * (var.offset * 0.05)
      text(p_v$x + off_x,
           p_v$y,
           labels=rownames(covar)[i],
           pos=var.pos,
           offset=var.offset,
           col=var.color[i],
           cex=var.cex)
    }

    if(obj.identify)
      identify(x=graph$xyz.convert(coobj),
               labels=obj.labels,
               cex=obj.cex)

    par(op)
  }

  # --- Modo RGL ---
  if(rgl.use) {
    size <- max(abs(coobj)) / 20 * obj.cex
    if (clear3d) rgl::clear3d()

    rgl::plot3d(scores,
                xlim=xlim,
                ylim=ylim,
                zlim=zlim,
                xlab='',
                ylab='',
                zlab='',
                type='n',
                axes=FALSE,
                box=box,
                aspect=aspect, ...)

    if (obj.names) {
      rgl::spheres3d(coobj,
                     col=obj.color,
                     radius=size / 2,
                     alpha=.5)

      rgl::text3d(coobj,
                  texts=obj.labels,
                  col=obj.color,
                  adj=obj.pos,
                  cex=obj.cex)
    } else {
      rgl::spheres3d(coobj,
                     col=obj.color,
                     radius=size,
                     alpha=.5)
    }

    for(i in 1:nrow(covar)) {
      v_curr <- covar[i,] * var.factor

      rgl::spheres3d(v_curr,
                     col=var.color[i],
                     radius=size / 3,
                     alpha=.5)

      rgl::segments3d(rbind(c(0,0,0),
                      v_curr),
                      col=var.color[i])

      off <- (obj.cex * 0.05)

      rgl::text3d(v_curr + sign(v_curr)*off,
                  texts=rownames(covar)[i],
                  col=var.color[i],
                  cex=var.cex)
    }

    if(simple.axes) {
      rgl::axes3d(c('x', 'y', 'z'))

      rgl::title3d(xlab=xlab,
                   ylab=ylab,
                   zlab=zlab)
    } else {
      rgl::decorate3d(xlab=xlab,
                      ylab=ylab,
                      zlab=zlab,
                      box=box)
    }
    if(ref.lines)
      rgl::abclines3d(0, 0, 0,
                      a=diag(3),
                      lty=ref.lty,
                      col=ref.color)
  }
}

# José Cláudio Faria
var.rbf <- function(x)
{
  lv <- function(x) sqrt(t(x) %*% x)  # length of vector
  l  <- apply(x, 1, lv)
  n  <- nrow(x)
  var.rb <- diag(n)

  for (i in 1:(n-1)) {
    for (j in (i+1):n) {

      cost <- (t(x[i,]) %*%
               x[j,]) /
      (l[i]*l[j])

      var.rb[j,i] <- cost         # fill lower.tri
      var.rb[i,j] <- var.rb[j,i]  # fill upper.tri
    }
  }

  dimnames(var.rb) <- list(dimnames(x)[[1]],
                           dimnames(x)[[1]])
  return(var.rb)
}

# José Cláudio Faria
bpca.prcomp <- function(x,
                        d=1:2, ...)
{
  stopifnot(class(x) == 'prcomp')

  if (!length(x$x))
    stop(gettextf("object '%s' has no objects coordinates!",
                  deparse(substitute(x))), domain=NA)

  if (is.complex(x$x))
    stop("bpca is not defined for complex PCA!")

  li <- d[1]
  le <- d[length(d)]
  n.lambda <- (le - li + 1)

  if(n.lambda < 2 || n.lambda > 3)
    stop('Please, check the parameter d:\n',
         'The (d[1] - d[length(d)] + 1) must equal to 2 (for bpca.2d) or 3 (for bpca.3d).\n\n')

  # Due to necessity of different type of factorization it will go back
  # from prcom, i.e, it will regenerate the data x already scaled!
  xreg <- x$x %*%
    (solve(t(x$rotation) %*%
           x$rotation) %*%
  t(x$rotation))
  # xreg <- x$x %*%
  # ginv(x$rotation) # another option (require MASS)
  bpca.default(xreg,
               d,
               center=ifelse(x$center[1] == FALSE, 0, 2),
               scale=ifelse(x$scale[1] == FALSE, 0, 1), ...)
}

# José Cláudio Faria
xtable.bpca <- function(x,
                        caption=NULL,
                        label=NULL,
                        align=NULL,
                        digits=NULL,
                        display=NULL,
                        auto=FALSE, ...)
{
  eigvec <- x$eigenvectors[, c(x$number)]
  eigval <- x$eigenvalues[c(x$number)]
  vret <- x$eigenvalues[x$number[1]:x$number[length(x$number)]]^2/sum(x$eigenvalues^2)
  vacum <- cumsum(vret)     

  x <- rbind(eigvec,
             eigval,
             vret,
             vacum)

  row.names(x) <- c(paste("Eigenvectors",
                          row.names(eigvec),
                          sep='\\_'), 
                    "Eigenvalues",
                    "Variance retained", 
                    "Variance accumulated")

  res <- xtable(x, 
                caption=caption, 
                label=label, 
                align=align, 
                digits=digits, 
                display=display, 
                auto=auto, ...)
  class(res) <- c("xtable.bpca","xtable","data.frame")
  return(res)
}

# José Cláudio Faria
bpca.default <- function(x,
                         d=1:2,
                         center=2,
                         scale=TRUE,
                         method=c('hj', 'sqrt', 'jk', 'gh'),
                         iec=FALSE,
                         var.rb=FALSE,
                         var.rd=FALSE,
                         limit=10, ...)
{
  stopifnot(is.matrix(x) || is.data.frame(x))

  li <- d[1]
  le <- d[length(d)]
  n.lambda <- (le - li + 1)

  if(n.lambda < 2 || n.lambda > 3)
    stop('Please, check the parameter d:\n',
         'The (d[1] - d[length(d)] + 1) must equal to 2 (for bpca.2d) or 3 (for bpca.3d).\n\n')

  x <- as.matrix(x)

  x.cent <- x                                             # 0: no centering
  switch(center,                                          # of course, if center=1:3
         x.cent <- sweep(x, 1, mean(x)),                  # 1: global-centered
         x.cent <- sweep(x, 2, apply(x, 2, mean)),        # 2: column-centered
         x.cent <- sweep(sweep(x, 1, apply(x, 1, mean)),  # 3: double-centered
                         2, apply(x, 2, mean)) + mean(x))

  if(scale)
    x.scal <- sweep(x.cent, 2, apply(x.cent, 2, sd), '/')
  else
    x.scal <- x.cent

  svdx.scal <- svd(x.scal)

  rownames(svdx.scal$u) <- rownames(x) # obj
  rownames(svdx.scal$v) <- colnames(x) # var
  colnames(svdx.scal$v) <- paste('PC',
                                 1:length(svdx.scal$d),
                                 sep='')

  s2.scal <- diag(svdx.scal$d)

  switch(match.arg(method),
         hj={
           g.scal  <- svdx.scal$u %*% s2.scal
           h.scal  <- s2.scal %*% t(svdx.scal$v)
           hl.scal <- t(h.scal)
         },
         sqrt={
           g.scal  <- svdx.scal$u %*% sqrt(s2.scal)
           h.scal  <- sqrt(s2.scal) %*% t(svdx.scal$v)
           hl.scal <- t(h.scal)
         },
         jk={
           g.scal  <- svdx.scal$u %*% s2.scal
           hl.scal <- svdx.scal$v

           ## The below is the GGEBiplot aproach
           #d1 <- (max(hl.scal[,li]) - min(hl.scal[,li])) /
           #(max(g.scal[,li]) - min(g.scal[,li]))
           #d2 <- (max(hl.scal[,le]) - min(hl.scal[,le])) /
           #(max(g.scal[,le]) - min(g.scal[,le]))
           #d <- max(d1,d2)

           #hl.scal / d
         },
         gh={
           g.scal  <- sqrt(nrow(x)-1) * svdx.scal$u
           h.scal  <- 1/sqrt(nrow(x)-1) * s2.scal %*% t(svdx.scal$v)
           hl.scal <- t(h.scal)

           ## The below is the GGEBiplot aproach
           #g.scal  <- svdx.scal$u
           #hl.scal  <- svdx.scal$v %*% s2.scal

           #d1 <- (max(g.scal[,li]) - min(g.scal[,li])) / 
           #(max(hl.scal[,li]) - min(hl.scal[,li]))
           #d2 <- (max(g.scal[,le]) - min(g.scal[,le])) / 
           #(max(hl.scal[,le]) - min(hl.scal[,le]))
           #d <- max(d1,d2)
           #g.scal <- g.scal / d
         })

  pc.names <- paste('PC',
                    1:ncol(hl.scal),
                    sep='')

  if(is.null(rownames(x.scal)))
    rownames(g.scal) <- 1:nrow(x.scal)
  else
    rownames(g.scal) <- rownames(x.scal)
  colnames(g.scal) <- pc.names
  if(is.null(colnames(x.scal)))
    rownames(hl.scal) <- paste('V',
                               1:ncol(x),
                               sep='')
  else
    rownames(hl.scal) <-colnames(x.scal)
  colnames(hl.scal) <- pc.names

  # variables
  if(var.rb)
    var.rb.res <- var.rbf(hl.scal[,d[1]:d[length(d)]])
  else
    var.rb.res <- NA

  if(var.rb & var.rd)
    var.rd.res <- var.rdf(x.scal, 
                          var.rb.res, 
                          limit)
  else
    var.rd.res <- NA

  if(iec){
    svdx.scal$v <- (-1) * svdx.scal$v
    g.scal      <- (-1) * g.scal     
    hl.scal     <- (-1) * hl.scal    
  }   

  res <- list(call=match.call(),
              eigenvalues=svdx.scal$d,
              eigenvectors=svdx.scal$v,
              number=seq(li, le, 1),
              importance=rbind(general=round(sum(svdx.scal$d[li:le]^2) /
                                             sum(svdx.scal$d^2), 3),
                               partial=round(sum(svdx.scal$d[li:le]^2) /
                                             sum(svdx.scal$d[li:length(svdx.scal$d)]^2), 3)),
              coord=list(objects=g.scal,
                         variables=hl.scal),
              var.rb=var.rb.res,
              var.rd=var.rd.res)

  colnames(res$importance) <- 'explained'

  if(n.lambda == 2)
    class(res) <- c('bpca.2d', 
                    'bpca', 
                    'list')
  else if(n.lambda == 3)
    class(res) <- c('bpca.3d', 
                    'bpca', 
                    'list')

  invisible(res)
}

# José Cláudio Faria
dt.tools <- function(x,
                     center=2,
                     scale=TRUE)
{
  stopifnot(is.matrix(x) || is.data.frame(x))

  bCol <- sapply(x,
                 is.numeric)

  x <- as.matrix(x[,bCol])

  n <- ncol(x)
  if (n < 2 )
    stop('x has less than two columns (variables)!')

  switch(center,                                     # of course, if center=1:3
         x <- sweep(x, 1, mean(x)),                  # 1: row mean centering
         x <- sweep(x, 2, apply(x, 2, mean)),        # 2: column mean centering
         x <- sweep(sweep(x, 1, apply(x, 1, mean)),  # 3: double-centering
                    2, apply(x, 2, mean)) + mean(x))

  if(scale)
    x <- sweep(x, 2, apply(x, 2, sd), '/')
  else
    x <- x

  lv <- function(x) sqrt(t(x) %*% x)  # length of vector
  l  <- apply(x,
              2,
              lv)
  r  <- diag(n)

  for (i in 1:(n - 1)) {
    for (j in (i + 1):n) {
      cost <- (t(x[,i]) %*%
               x[,j]) /
      (l[i] * l[j])

    r[j,i] <- cost    # fill lower.tri
    r[i,j] <- r[j,i]  # fill upper.tri
    }
  }

  a <- acos(r) * 180 / pi

  dimnames(r) <- list(colnames(x),
                      colnames(x))

  dimnames(a) <- dimnames(r)

  res <- list(length=l,
              angle=a,
              r=r)

  invisible(res)
}

% José Cláudio Faria
%\VignetteIndexEntry{LaTeX tables for bpca objects}
%\VignetteEngine{knitr::knitr_notangle}
%\VignetteEncoding{UTF-8}

\documentclass[11pt,a4paper]{article}

\usepackage[english]{babel}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage[sc]{mathpazo}
\usepackage{microtype}
\usepackage{xcolor}
\usepackage{geometry}
\usepackage{titlesec}

\geometry{
  a4paper,
  tmargin=2.5cm,
  bmargin=2.5cm,
  lmargin=2.5cm,
  rmargin=2.5cm
}

\definecolor{accent}{RGB}{26, 82, 118}
\definecolor{rulegray}{RGB}{180, 180, 180}
\definecolor{titlemuted}{RGB}{90, 90, 90}

\usepackage[
  colorlinks=true,
  linkcolor=accent,
  citecolor=accent,
  urlcolor=accent
]{hyperref}
\hypersetup{
  pdftitle={LaTeX tables for bpca objects},
  pdfauthor={José C. Faria, Ivan B. Allaman}
}

\usepackage{url}
\usepackage{graphicx}
\usepackage{longtable}
\usepackage{float}
\usepackage{parskip}
\usepackage[round]{natbib}

% Required by latex.bpca
\usepackage{multirow}
\usepackage{colortbl}
\usepackage{array}
\usepackage{Sweave}

\setlength{\parindent}{0pt}
\setlength{\parskip}{0.65em}

\titleformat{\section}
  {\normalfont\Large\bfseries\color{accent}}
  {\thesection}{0.75em}{}
\titlespacing*{\section}{0pt}{2.25ex plus .5ex minus .2ex}{1.25ex plus .2ex}

\titleformat{\subsection}
  {\normalfont\large\bfseries\color{titlemuted}}
  {\thesubsection}{0.65em}{}

\newcommand{\HRule}{%
  \leavevmode\leaders\hrule height 0.6pt\hfill\kern0pt\relax}
\newcommand{\titleaccentline}{%
  {\color{accent}\rule{\linewidth}{1.4pt}}}

\begin{document}

\begin{titlepage}
  \centering
  \vspace*{1.2cm}

  {\color{rulegray}\HRule}
  \vspace{0.55cm}

  {\Huge\bfseries\color{accent}%
    \LaTeX\ tables for \texttt{bpca} objects}

  \vspace{0.35cm}
  {\large\color{titlemuted}%
    Practical examples with \texttt{xtable} and clean formatting}

  \vspace{0.55cm}
  {\color{rulegray}\HRule}

  \vspace{2.4cm}

  \begin{minipage}[t]{0.48\textwidth}
    \raggedright
    \textbf{\color{titlemuted}Authors}\\[0.35em]
    José C.\ \textsc{Faria}\\
    Ivan B.\ \textsc{Allaman}
  \end{minipage}%
  \hfill
  \begin{minipage}[t]{0.48\textwidth}
    \raggedleft
    \textbf{\color{titlemuted}\LaTeX\ customization}\\[0.35em]
    José C.\ \textsc{Faria}
  \end{minipage}

  \vfill

  \titleaccentline
  \vspace{0.6cm}
  {\large\today}
\end{titlepage}

<<setup, include=FALSE>>=
knitr::opts_chunk$set(
  results = "asis",
  echo = TRUE,
  comment = NA
)
@

\vspace{1.25cm}
\tableofcontents
\newpage

\section{Quick start}

This vignette presents a simple and reproducible workflow to generate
publication-ready \LaTeX\ tables from \texttt{bpca} objects:

\begin{enumerate}
  \item Fit a biplot model with \texttt{bpca()}.
  \item Convert it to an \texttt{xtable} object.
  \item Use \texttt{print()} options to refine labels and presentation.
\end{enumerate}

\subsection{Most used \texttt{print.xtable} arguments}

\begin{center}
\begin{tabular}{p{0.36\linewidth} p{0.55\linewidth}}
\hline
\textbf{Argument} & \textbf{Purpose} \\
\hline
\texttt{caption} & Add a descriptive caption to reference in text. \\
\texttt{label} & Create a \LaTeX\ label for cross-referencing (\texttt{\textbackslash ref\{\}}). \\
\texttt{sanitize.colnames.function} & Customize the rendering of column labels (for example, bold text). \\
\texttt{sanitize.rownames.function} & Customize row labels (for example, italic text). \\
\hline
\end{tabular}
\end{center}

\vspace{1.25cm}

<<libraries, message=FALSE, warning=FALSE>>=
library(bpca)
library(xtable)
@

\section{First table}

Start with a minimal example using the \texttt{iris} data.

<<simple-example>>=
bp <- bpca(iris[-5])
tb_simple <- xtable(bp)

## Build with xtable and print with print() so the bpca S3 method is used.
print(tb_simple)
@

\section{Cross-referencing tables}

Captions and labels make documents easier to navigate.
The following examples are referenced as Table \ref{tbl_iris} and
Table \ref{tbl_gabriel}.

<<cross-ref-iris>>=
tb_iris <- xtable(
  bpca(iris[-5]),
  caption = "Biplot of iris data (package: datasets).",
  label = "tbl_iris"
)
print(tb_iris)
@

<<cross-ref-gabriel>>=
tb_gabriel <- xtable(
  bpca(gabriel1971),
  caption = "Biplot of gabriel1971 data (package: bpca).",
  label = "tbl_gabriel"
)
print(tb_gabriel)
@

\section{Formatting for readability}

\subsection{Bold column names}

Table \ref{tbl_rock} highlights column names in bold.

<<bold-columns>>=
tb_rock <- xtable(
  bpca(rock),
  caption = "Biplot of rock data (package: datasets).",
  label = "tbl_rock"
)

bold <- function(x) paste0("\\textbf{", x, "}")

print(tb_rock, sanitize.colnames.function = bold)
@

\subsection{Italic row names}

Table \ref{tbl_usarrests} highlights row names in italic.

<<italic-rows>>=
tb_usarrests <- xtable(
  bpca(USArrests),
  caption = "Biplot of USArrests data (package: datasets).",
  label = "tbl_usarrests"
)

italic <- function(x) paste0("\\textit{", x, "}")

print(tb_usarrests, sanitize.rownames.function = italic)
@

\section{Localization of labels}

You can adapt labels to Portuguese and keep the same table structure.
Table \ref{tbl_rock_pt} shows this approach.

<<portuguese-labels>>=
tb_rock_pt <- xtable(
  bpca(rock),
  caption = "Biplot of rock data (package: datasets), Portuguese labels.",
  label = "tbl_rock_pt"
)

rownames(tb_rock_pt) <- gsub(
  "Eigenvalues",
  "Autovalores",
  rownames(tb_rock_pt)
)
rownames(tb_rock_pt) <- gsub(
  "Eigenvectors",
  "Autovetores",
  rownames(tb_rock_pt)
)
rownames(tb_rock_pt) <- gsub(
  "Variance retained",
  "Variância retida",
  rownames(tb_rock_pt)
)
rownames(tb_rock_pt) <- gsub(
  "Variance accumulated",
  "Variância acumulada",
  rownames(tb_rock_pt)
)

colnames(tb_rock_pt) <- c("CP1", "CP2")

print(tb_rock_pt)
@

\section{Custom print options in one call}

It is often convenient to customize row and column formatting directly in
a single \texttt{print()} call (Table \ref{tbl_directly}).

<<custom-print>>=
print(
  xtable(
    bpca(rock),
    caption = "Direct customization using print() on an xtable.bpca object.",
    label = "tbl_directly"
  ),
  sanitize.colnames.function = bold,
  sanitize.rownames.function = italic
)
@

\section{Takeaways}

\begin{itemize}
  \item Use \texttt{xtable(bpca(...))} + \texttt{print()} as the default workflow.
  \item Add \texttt{caption} and \texttt{label} early when writing reports.
  \item Use sanitization functions to improve readability (bold/italic, localized labels).
  \item Keep examples compact and reproducible to simplify maintenance.
\end{itemize}

\end{document}

---
title: "bpca Overview"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{bpca Overview}
  %\VignetteEngine{knitr::rmarkdown_notangle}
  %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

`bpca` is an S3-based package for principal component biplot analysis.
It supports:

- 2D and 3D biplots;
- multiple factorization methods (`hj`, `sqrt`, `jk`, `gh`);
- graphical diagnostics for representation quality;
- tabular summaries for reporting.

This vignette presents a practical end-to-end workflow.

## Load Package and Data

```{r}
library(bpca)
data(gabriel1971)
data(ontario)
```

`gabriel1971` is a classic biplot matrix and works well for reproducible
examples.

## 2D Biplot: Core Workflow

```{r}
bp2 <- bpca(gabriel1971,
  d = 1:2
)
class(bp2)
bp2$number
round(bp2$importance, 3)
```

Key returned components include:

- `eigenvalues`, `eigenvectors`;
- `number` (selected principal components);
- `importance` (retained variance);
- `coord$objects` and `coord$variables`.

```{r}
names(bp2)
str(bp2$coord)
```

## Summary Output

```{r}
summary(bp2)
summary(bp2,
  presentation = TRUE
)
```

## Factorization Methods

```{r}
methods <- c("hj", "sqrt", "jk", "gh")
retained <- sapply(
  methods,
  function(m) {
    bp <- bpca(gabriel1971,
      method = m,
      d = 1:2
    )
    as.numeric(bp$importance["general", "explained"])
  }
)
retained
```

The retained variance can be similar across methods, but geometric
interpretation differs.

## Representation Quality Diagnostics

`var.rb` stores projected correlations among variables under the selected
biplot reduction.

```{r}
bp2_q <- bpca(gabriel1971,
  d = 1:2,
  var.rb = TRUE
)
round(bp2_q$var.rb, 2)
```

`qbpca()` compares observed vs projected correlations in vectorized form.

```{r}
q2 <- qbpca(
  gabriel1971,
  bp2_q
)
head(q2)
```

`var.rd` highlights poorly represented variable correlations for a chosen
threshold (`limit`).

```{r}
bp2_rd <- bpca(gabriel1971,
  d = 1:2,
  var.rb = TRUE,
  var.rd = TRUE,
  limit = 3
)
bp2_rd$var.rd
```

## 3D Workflow

```{r}
bp3 <- bpca(gabriel1971,
  d = 1:3,
  var.rb = TRUE
)
bp3$number
round(bp3$importance, 3)
```

## Plot Types and Examples

The `plot.bpca.2d` method supports several analysis-oriented views via `type`:

- `"bp"`: basic biplot;
- `"eo"`: evaluate one object;
- `"ev"`: evaluate one variable;
- `"co"` / `"cv"`: compare objects / variables;
- `"ww"`, `"dv"`, `"ms"`, `"ro"`, `"rv"` for GGE-style diagnostics.

Examples (kept as non-evaluated chunks to keep vignette deterministic):

```{r, eval=FALSE}
# Basic 2D
plot(bp2)

# Evaluate one object
plot(bpca(ontario),
  type = "eo",
  obj.id = 7
)

# Evaluate one variable
plot(bpca(ontario),
  type = "ev",
  var.id = "E7"
)

# Compare two objects
plot(bpca(ontario),
  type = "co",
  obj.id = c("g7", "g13")
)
```

3D rendering can be static (`scatterplot3d`) or interactive (`rgl`):

```{r, eval=FALSE}
# Static 3D
plot(bp3)

# Interactive 3D
plot(bp3,
  rgl.use = TRUE
)
```

## Tabular Reporting with xtable

The package provides an `xtable` method for `bpca` objects.

```{r}
if (requireNamespace("xtable", quietly = TRUE)) {
  tbl <- xtable::xtable(bp2)
  tbl[1:6, , drop = FALSE]
}
```

Use **`print()`** (S3) so the `xtable.bpca` formatter runs; `xtable::print.xtable()`
does not dispatch to it.

### HTML (this vignette)

For HTML output (including this `html_vignette`), pass `type="html"`. The chunk
must use `results='asis'` so the table is inserted as raw HTML.

```{r, results="asis"}
if (requireNamespace("xtable",
  quietly = TRUE
)) {
  tbl <- xtable::xtable(bp2)
  print(tbl,
    type = "html"
  )
}
```

### LaTeX (PDF reports)

To generate the biplot-style LaTeX table (e.g. `pdf_document` or Sweave), use
the default `type` and call `print()`:

```{r, eval=FALSE}
tbl <- xtable::xtable(bp2,
  caption = "Biplot summary",
  label = "tab:bp2"
)
rn <- rownames(tbl)
idx_retained <- which(rn == "Variance retained")
hlines <- c(-1, 0, nrow(tbl) - 1L)
if (length(idx_retained) == 1L && idx_retained > 1L) {
  hlines <- c(hlines, idx_retained - 1L)
}
hlines <- sort(unique(hlines))
print(tbl,
  hline.after = hlines
)
```

## Notes

- Use `d=1:2` for 2D and `d=1:3` for 3D.
- Prefer `var.rb=TRUE` and `var.rd=TRUE` when diagnosis quality is part
  of your analysis workflow.
- For scripts and reproducibility, prefer explicit package namespaces in
  custom reporting code (`xtable::`, `rgl::`).

\name{marina}
\alias{marina}

\title{
  Films shown at five festivals in Brazil from 2007 to 2011
}

\description{
  A data.frame containing the films shown at five festivals in Brazil from 2007 to 2011.
}

\usage{
  data(marina)
}

\format{
  The format is:
  \verb{
    'data.frame': 25 obs. of  6 variables:
    year: int  2011 2011 2011 2011 2011 2010 2010 2010 2010 2010 ...
    regE: Factor w/ 5 levels "CO","N","NE",..: 1 5 4 3 2 1 5 4 3 2 ...
    F   : int  84 55 63 44 25 40 54 37 49 27 ...
    D   : int  26 13 19 16 7 9 14 11 19 6 ...
    MD  : int  22 9 13 14 5 6 11 10 13 3 ...
    WD  : int  4 4 6 2 2 3 3 1 6 3 ...
    #
    # Description
    #
    year: Year in which the film was shown
    regE: Region where the film was shown
    F   : Total number of films
    D   : Number of documentaries
    MD  : Documentary directed by men
    WD  : Documentary directed by woman
  }
}

\examples{
data(marina)

marina
}

\keyword{datasets}
\keyword{movie}

\name{var.rdf}
\alias{var.rdf}

\title{
  Diagnostic of Projected Correlations
}

\description{
  Computes the diagnostic of poor graphical correlations projected by biplot
  according to an arbitrary \samp{limit}.
}

\usage{
  var.rdf(x,
          var.rb,
          limit)
}

\arguments{
  \item{x}{A given object of the classe \code{data.frame} or \code{matrix}.}
  \item{var.rb}{A given object of the class \code{matrix} with the projected correlations by biplot.}
  \item{limit}{A vector giving the percentual limit to define poor representation of variables.}
}

\value{
  A \code{data.frame} of poor graphical correlations projected by biplot.
}

\note{
  This function is mainly for internal use in the \pkg{bpca} package,
  and may not remain available (unless we see a good reason).
}

\author{
  Faria, J. C. \cr
  Allaman, I. B. \cr
  Demétrio C. G. B.
}

\seealso{\code{\link{bpca}}.}

\examples{
##
## Example 1
## Diagnostic of gabriel1971 dataset representation
##

oask <- devAskNewPage(dev.interactive(orNone=TRUE))

bp1 <- bpca(gabriel1971,
            meth='hj',
            var.rb=TRUE)

(res <- var.rdf(gabriel1971,
                bp1$var.rb,
                lim=3))
class(res)

##
## Example 2
## Diagnostic of gabriel1971 dataset representation with var.rd parameter
##

bp2 <- bpca(gabriel1971,
            meth='hj',
            var.rb=TRUE,
            var.rd=TRUE,
            limit=3)

plot(bp2,
     var.factor=2)

bp2$var.rd

bp2$eigenvectors

# Graphical visualization of the importance of the variables not contemplated
# in the reduction
plot(bpca(gabriel1971,
          meth='hj',
          d=3:4),
     main='hj')

# Interpretation:
# RUR followed by CRISTIAN contains information dimensions that
# wasn't contemplated by the biplot reduction (PC3).
# Between all, RUR followed by CRISTIAN, variables are the most poor represented
# by a 2d biplot.

\dontrun{
##
## Example 3
## Diagnostic of iris dataset representation with var.rd parameter
##

bp3 <- bpca(iris[-5],
            var.rb=TRUE,
            var.rd=TRUE,
            limit=3)

plot(bp3,
     obj.col=c('red', 'green3', 'blue')[as.numeric(iris$Species)],
     var.factor=.3)

bp3$var.rd
bp3$eigenvectors

# Graphical diagnostic
plot(bpca(iris[-5],
          d=3:4),
     obj.col=c('red', 'green3', 'blue')[as.numeric(iris$Species)],
     obj.names=FALSE)

# Interpretation:
# Sepal.length followed by Petal.Width contains information in dimensions
# (PC3 - the PC3 is, essentially, a contrast among both) that wasn't fully
# contemplated by the biplot reduction (PC1 and PC2) .
# Therefore, between all variables, they have the most poor representation by a
# 2d biplot.

bp4 <- bpca(iris[-5],
            d=1:3,
            var.rb=TRUE,
            var.rd=TRUE,
            limit=2)

plot(bp4,
     obj.names=FALSE,
     obj.pch=c('+', '-', '*')[as.numeric(iris$Species)],
     obj.col=c('red', 'green3', 'blue')[as.numeric(iris$Species)],
     obj.cex=1)

bp4$var.rd
bp4$eigenvectors

round(bp3$var.rb, 2)

round(cor(iris[-5]), 2)

# Good representation of all variables with a 3d biplot!
}

devAskNewPage(oask)
}

\keyword{multivariate}

\name{plot}
\alias{plot.bpca.2d}
\alias{plot.bpca.3d}
\alias{plot.qbpca}

\title{
  Biplot of Multivariate Data Based on Principal Components Analysis
}

\description{
  These are methods for objects of class \code{bpca.2d}, \code{bpca.3d} and
  \code{qbpca}.
}

\usage{
  \method{plot}{bpca.2d}(x,
     type=c('bp', 'eo', 'ev', 'co', 'cv', 'ww', 'dv', 'ms', 'ro', 'rv'),
     c.color='darkgray',
     c.lwd=1,
     c.number=5,
     c.radio=1,
     obj.id=1:2,
     var.id=1,
     base.color='red3',
     base.lty='dotted',
     proj.color='gray',
     proj.lty='dotted',
     a.color='blue',
     a.lty='solid',
     a.lwd=2,
     a.length=.1,
     ref.lines=TRUE,
     ref.color='navy',
     ref.lty='dotted',
     var.factor=NULL,
     var.color='red3',
     var.lty='solid',
     var.pch=20,
     var.pos=4,
     var.cex=.6,
     var.offset=.2,
     obj.factor=1,
     obj.color='black',
     obj.pch=20,
     obj.pos=4,
     obj.cex=.6,
     obj.offset=.2,
     obj.names=TRUE,
     obj.labels,
     obj.identify=FALSE,
     xlim,
     ylim,
     xlab,
     ylab, \dots)

  \method{plot}{bpca.3d}(x,
     rgl.use=FALSE,
     ref.lines=TRUE,
     ref.color='navy',
     ref.lty=ifelse(rgl.use, NA, 'dotted'),
     clear3d=ifelse(rgl.use, TRUE, NULL),
     simple.axes=ifelse(rgl.use, TRUE, NULL),
     aspect=ifelse(rgl.use, c(1, 1, 1), NULL),
     var.factor=NULL,
     var.color='red3',
     var.lty=ifelse(rgl.use, NA, 'solid'),
     var.pch=ifelse(rgl.use, NULL, 20),
     var.pos=ifelse(rgl.use, 0, 4),
     var.cex=ifelse(rgl.use, .8, .6),
     var.offset=ifelse(rgl.use, NULL, .2),
     obj.color='black',
     obj.pch=ifelse(rgl.use, NULL, 20),
     obj.pos=ifelse(rgl.use, 0, 4),
     obj.cex=ifelse(rgl.use, .8, .6),
     obj.offset=ifelse(rgl.use, NULL, .2),
     obj.names=TRUE,
     obj.labels,
     obj.identify=FALSE,
     box=FALSE,
     angle=ifelse(rgl.use, NULL, 40),
     xlim,
     ylim,
     zlim,
     xlab,
     ylab,
     zlab, \dots)

  \method{plot}{qbpca}(x,
     xlab='Index',
     ylab='r',
     pch=c(1,8),
     col=c(4,2), \dots)
}

\arguments{
  \item{x}{A \code{bpca.2d}, \code{bpca.3d} or \code{qbpca} object.}
  \item{type}{Type of biplot: \cr
              \samp{bp} - conventional biplot; \cr
              \samp{eo} - evaluate an object; \cr
              \samp{ev} - evaluate a variable; \cr
              \samp{co} - compare two objects; \cr
              \samp{cv} - compare (relationship among) variables; \cr
              \samp{ww} - which won where/what; \cr
              \samp{dv} - discrimitiveness vs. representativeness; \cr
              \samp{ms} - mean vs. stability; \cr
              \samp{ro} - rank objects with ref. to an \dQuote{ideal} variable; \cr
              \samp{rv} - rank variables with ref. to an \dQuote{ideal} object.}
  \item{c.color}{The color of circles.}
  \item{c.lwd}{The \code{lwd} of circles.}
  \item{c.number}{The number of circles.}
  \item{c.radio}{The radio of circles.}
  \item{obj.id}{An object(s) number(s) or name(s).
    It is used with reprojctions to identify the object(s) when the \code{type} option is \samp{'eo'} or \samp{'co'}.
    The default is \samp{1} for \samp{'eo'} and \code{1:2} for \samp{'co'}.}
  \item{var.id}{A variable number or name.
    It is used with reprojections to identify the variable when the \samp{type} option is \code{'ev'}.}
  \item{base.color}{The color for the base lines.
    It is used only with reprojections.}
  \item{base.lty}{The \samp{lty} for the base lines.
    It is used only with reprojections.}
  \item{proj.color}{The color for the projections lines.
    It is used only with reprojections.}
  \item{proj.lty}{The \samp{lty} for the projections lines.
    It is used only with reprojections.}
  \item{a.color}{The color for the arrow.
    It is used only with reprojections.}
  \item{a.lty}{The \samp{lty} for the arrow.
    It is used only with reprojections.}
  \item{a.lwd}{The \samp{lwd} for the arrow.
    It is used only with reprojections.}
  \item{a.length}{The \samp{length} for the arrow.
    It is used only with reprojections.}
  \item{rgl.use}{A logical value.
    If \code{TRUE} the 3d scatter will be done with the \code{rgl} environment,
    in another way the \code{scatterplot3d} will be used.}
  \item{ref.lines}{A logical value.
    If \code{TRUE}, the reference lines will be added to the biplot.}
  \item{ref.color}{Line color for reference lines.}
  \item{ref.lty}{Line type of the reference lines.}
  \item{clear3d}{A logical value.
    It clears (or not) a 3d biplot before making a new one.
    Used only if \code{rgl.use=TRUE}.}
  \item{simple.axes}{A logical value to draw simple axes.
    Used only if \code{rgl.use=TRUE}.}
  \item{aspect}{A vector of the apparent ratios of the \samp{x}, \samp{y},
    and \samp{z} axes of the bounding box. Used only if \code{rgl.use=TRUE}.}
  \item{var.factor}{Factor of expansion/reduction of length lines of the variables.}
  \item{var.color}{A vector of colors for the variables representation.}
  \item{var.lty}{Line type for the variables.
    Used only if \code{rgl.use=FALSE}.}
  \item{var.pch}{A vector of plotting symbols or characters for the variables.
    If too short, the values are recycled.
    Used only if \code{rgl.use=FALSE}.}
  \item{var.pos}{Position of labels for the variables.}
  \item{var.cex}{Character expansion for the variables.}
  \item{var.offset}{The distance (in character widths) which separates the
    label from identified points of variables.
    Negative values are allowed.
    Used only if \code{rgl.use=FALSE}.}
  \item{obj.factor}{Factor of expansion/reduction of length lines of the objects.}
  \item{obj.color}{A vector of colors for the objects representation.}
  \item{obj.pch}{A vector of plotting symbols or characters for objects.
    If too short, the values are recycled.
    Used only if \code{rgl.use=FALSE}.}
  \item{obj.pos}{Position of labels for objects.}
  \item{obj.cex}{Character expansion for objects.}
  \item{obj.offset}{The distance (in character widths) which separates the label
    from identified points of objects. Negative values are allowed.
    Used only if \code{rgl.use=FALSE}.}
  \item{obj.names}{A logical value to represent objects as spheres or points.}
  \item{obj.identify}{A logical value.
    If \code{TRUE}, it is possible, in interative way, to identify the objects with the mouse.}
  \item{obj.labels}{A vector of labels for objects.}
  \item{box}{A logical value to whether to draw a box.
    Used only if \samp{rgl.use=TRUE}.}
  \item{angle}{Angle between \samp{x} and \samp{y} axis (Attention: result depends on scaling).
    For \eqn{180 < angle < 360} the returned functions xyz.convert and points3d will not work properly.
    Used only if \code{rgl.use=FALSE}.}
  \item{pch}{A vector of plotting symbols or characters.}
  \item{col}{A vector of colors.}
  \item{xlab}{A label for the \samp{x} axis.}
  \item{ylab}{A label for the \samp{y} axis.}
  \item{zlab}{A label for the \samp{z} axis (bpca.3d only).}
  \item{xlim}{The \samp{x} limits of the plot.}
  \item{ylim}{The \samp{y} limits of the plot.} 
  \item{zlim}{The \samp{z} limits of the plot (bpca.3d only).} 
  \item{\dots}{Other graphical parameters may also be passed as arguments to these functions.}
}

\details{
  A biplot aims to represent both the observations and variables
  of a matrix of multivariate data on the same plot.

  The methods \code{plot.bpca.2d} draw a 2d biplot (PC1 and PC2 on axis \samp{x}
  and \samp{y}, respectively). It uses the traditional graphics system.

  The methods \code{plot.bpca.3d} draw a 3d biplot (PC1, PC2 and PC3 on axis
  \samp{x}, \samp{y} and \samp{z}, respectively) in two ways:

  \enumerate{
    \item static: It uses the package \code{scatterplot3d} based on
      traditional graphic system. Use the parameter \samp{rgl.use=FALSE} for it.
      It is the default.
    \item dinamic: It uses the package \code{rgl} a 3D real-time rendering
      device driver system for R. Use the parameter \samp{rgl.use=TRUE} for it.
  }

  The method \code{qb.pca} draws a scatterplot of the correlations observed
  (in the data) and projected (under the biplot) related to the variables.
  It uses the traditional graphics system.
  \code{qb.pca} is a simple (and graphical) measure of the quality of the biplot
  reduction associated to the variables.
}

\author{
  Faria, J. C. \cr
  Allaman, I. B. \cr
  Demétrio C. G. B.
}

\references{
  Gabriel, K. R. (1971)
  The biplot graphical display of matrices with application to principal
  component analysis.
  \emph{Biometrika} \bold{58}, 453-467.

  Galindo Vilardón, M. P. (1986)
  Una alternativa de representación simultánea: HJ-Biplot.
  \emph{Qüestiió}, 10(1):13-23, 1986.

  Johnson, R. A. and Wichern, D. W. (1988)
  \emph{Applied multivariate statistical analysis}.
  Prentice-Hall, Inc., Upper Saddle River, NJ, USA, 6 ed.

  Gower, J.C. and Hand, D. J. (1996)
  \emph{Biplots}. Chapman & Hall.

  Yan, B. W. and Kang, M. S. (2003)
  \emph{GGE biplot analysis: a graphical tool for breeders,
  geneticists, and agronomists}. CRC Press, New York, 288p.
}

\seealso{\code{\link{biplot.prcomp}}}

\examples{
##
## Example 1
## Computing and ploting a bpca object with 'graphics' package - 2d
##

bp <- bpca(gabriel1971)

dev.new(w=6, h=6)
oask <- devAskNewPage(dev.interactive(orNone=TRUE))
plot(bp)

# Additional graphical parameters (nonsense)
plot(bpca(gabriel1971,
          meth='sqrt'),
     main='gabriel1971 - sqrt',
     sub='The graphical parameters are working fine!',
     var.cex=.6,
     var.col=rainbow(9),
     var.pch='v',
     obj.pch='o',
     obj.cex=.5,
     obj.col=rainbow(8),
     obj.pos=1,
     obj.offset=.5)

##
## Example 2
## Computing and plotting a bpca object with 'scatterplot3d' package - 3d
##

bp <- bpca(gabriel1971,
           d=1:3)

plot(bp,
     var.col=rainbow(9))

# Additional graphical parameters (nonsense)
plot(bpca(gabriel1971,
          d=1:3,
          meth='jk'),
     main='gabriel1971 - jk',
     sub='The graphical parameters are working fine!',
     var.pch='+',
     var.cex=.6,
     var.col=rainbow(ncol(gabriel1971)),
     obj.pch='*',
     obj.cex=.8,
     obj.col=rainbow(nrow(gabriel1971)),
     ref.lty='dotted',
     ref.col=gray(.6),
     angle=70)

##
## Example 3
## Computing and plotting a bpca object with 'rgl' package - 3d
##

plot(bpca(gabriel1971,
          d=1:3),
     rgl.use=TRUE)

# Suggestion: Interact with the graphic with the mouse
# left button: press, maintain and movement it to interactive rotation;
# right button: press, maintain and movement it to interactive zoom.
# Enjoy it!

\dontrun{
##
## Example 4
## Grouping objects with different symbols and colors - 2d and 3d
##

# 2d
plot(bpca(iris[-5]),
     var.cex=.7,
     obj.names=FALSE,
     obj.cex=1.5,
     obj.col=c('red', 'green3', 'blue')[as.numeric(iris$Species)],
     obj.pch=c('+', '*', '-')[as.numeric(iris$Species)])

# 3d static
plot(bpca(iris[-5],
          d=1:3),
     var.color=c('blue', 'red'),
     var.cex=1,
     obj.names=FALSE,
     obj.cex=1,
     obj.col=c('red', 'green3', 'blue')[as.numeric(iris$Species)],
     obj.pch=c('+', '*', '-')[as.numeric(iris$Species)])

# 3d dynamic
plot(bpca(iris[-5],
          method='hj',
          d=1:3),
     rgl.use=TRUE,
     var.col=c('blue', 'red'),
     var.cex=1.2,
     obj.names=FALSE,
     obj.cex=.8,
     obj.col=c('red', 'green3', 'orange')[as.numeric(iris$Species)],
     simple.axes=FALSE,
     box=TRUE)

##
## Example 5
## Computing and plotting a bpca object with 'obj.identify=TRUE' parameter - 2d
##

bp <- bpca(gabriel1971)

# Normal labels
if(interactive())
plot(bp,
     obj.names=FALSE,
     obj.identify=TRUE)

# Alternative labels
if(interactive())
plot(bp,
     obj.names=FALSE,
     obj.labels=c('toi', 'kit', 'bat', 'ele', 'wat', 'rad', 'tv', 'ref'),
     obj.identify=TRUE)

##
## Example 6
## Computing and plotting a bpca object with 'obj.identify=TRUE' parameter - 3d
##

bp <- bpca(gabriel1971,
           d=1:3)

# Normal labels
if(interactive())
plot(bp,
     obj.names=FALSE,
     obj.identify=TRUE)

# Alternative labels
if(interactive())
plot(bp,
     obj.names=FALSE,
     obj.labels=c('toi', 'kit', 'bat', 'ele', 'wat', 'rad', 'tv', 'ref'),
     obj.identify=TRUE)

##
## New options plotting
##
plot(bpca(ontario))

# Labels for all objects
(obj.lab <- paste('g',
                  1:18,
                  sep=''))

# Giving obj.labels
plot(bpca(ontario),
    obj.labels=obj.lab) 

# Evaluate an object (1 is the default)
plot(bpca(ontario),
     type='eo',
     obj.cex=1)

plot(bpca(ontario),
     type='eo',
     obj.id=7,
     obj.cex=1)

# Giving obj.labels
plot(bpca(ontario),
     type='eo',
     obj.labels=obj.lab,
     obj.id=7,
     obj.cex=1)

# The same as above
plot(bpca(ontario),
     type='eo',
     obj.labels=obj.lab,
     obj.id='g7',
     obj.cex=1)

# Evaluate a variable (1 is the default)
plot(bpca(ontario),
     type='ev',
     var.pos=2,
     var.cex=1)

plot(bpca(ontario),
     type='ev',
     var.id='E7',
     obj.labels=obj.lab,
     var.pos=1,
     var.cex=1)

# A complete plot
cl <- 1:3
plot(bpca(iris[-5]),
     type='ev',
     var.id=1,
     var.fac=.3,
     obj.names=FALSE,
     obj.col=cl[as.numeric(iris$Species)])
legend('topleft',
       legend=levels(iris$Species),
       text.col=cl,
       pch=19,
       col=cl,
       cex=.9,
       box.lty=0)   

# Compare two objects (1 and 2 are the default)
plot(bpca(ontario), 
     type='co',
     c.radio=.4,
     c.color='blue',
     c.lwd=2)

plot(bpca(ontario),
     type='co',
     obj.labels=obj.lab,
     c.radio=.5,
     c.color='blue',
     c.lwd=2)

plot(bpca(ontario),
     type='co',
     obj.labels=obj.lab,
     obj.id=13:14)

plot(bpca(ontario),
     type='co',
     obj.labels=obj.lab,
     obj.id=c('g7', 
              'g13'))

# Compare two variables
plot(bpca(ontario),
     type='cv',
     c.number=3,
     c.radio=1.5)

# Which won where/what
plot(bpca(ontario),
     type='ww')

# Discrimitiveness vs. representativeness
plot(bpca(ontario),
     type='dv')

plot(bpca(ontario),
     type='dv',
     c.number=4,
     c.radio=1)

# Means vs. stability
plot(bpca(ontario),
     type='ms')

plot(bpca(ontario),
     type='ms',
     c.number=3)

# Rank objects with ref. to the ideal variable 
plot(bpca(ontario),
     type='ro')

plot(bpca(ontario),
     type='ro',
     c.number=6,
     c.radio=.5)

# Rank variables with ref. to the ideal object
plot(bpca(ontario),
     type='rv')

plot(bpca(ontario),
     type='rv',
     c.number=6,
     c.radio=.5)

plot(bpca(iris[-5]),
     type='eo',
     obj.id=42,
     obj.cex=1)

plot(bpca(iris[-5]),
     type='ev',
     var.id='Sepal.Width')

plot(bpca(iris[-5]),
     type='ev',
     var.id='Sepal.Width',
     var.factor=.3)
}

devAskNewPage(oask)
}

\keyword{multivariate}

\name{bpca-package}
\alias{bpca-package}
\docType{package}

\title{
  Biplot of Multivariate Data Based on Principal Components Analysis
}

\description{
  Implements biplot (2d and 3d) and diagnostic tools of the quality of the reduction.
}

\author{
  Faria, J. C. \cr
  Allaman, I. B. \cr
  Demétrio C. G. B.
}

\references{
  Gabriel, K. R.  (1971)
  The biplot graphical display of matrices with application to principal
  component analysis.
  \emph{Biometrika} \bold{58}, 453-467.

  Galindo Vilardón, M. P. (1986)
  Una alternativa de representación simultánea: HJ-Biplot.
  \emph{Qüestiió}, 10(1):13-23, 1986.

  Johnson, R. A. and Wichern, D. W. (1988)
  \emph{Applied multivariate statistical analysis}.
  Prentice-Hall, Inc., Upper Saddle River, NJ, USA, 6 ed.

  Gower, J.C. and Hand, D. J.  (1996)
  \emph{Biplots}. Chapman & Hall.

  Yan, B. W. and Kang, M. S. (2003)
  \emph{GGE biplot analysis: a graphical tool for breeders,
  geneticists, and agronomists}. CRC Press, New York, 288p.
}

\keyword{package}
\keyword{multivariate}

\examples{
##
## Grouping objects with different symbols and colors - 2d and 3d
##

dev.new(w=6, h=6)
oask <- devAskNewPage(dev.interactive(orNone=TRUE))

\dontrun{
# 2d
plot(bpca(iris[-5]),
     var.cex=.7,
     obj.names=FALSE,
     obj.cex=1.5,
     obj.col=c('red', 'green3', 'blue')[as.numeric(iris$Species)],
     obj.pch=c('+', '*', '-')[as.numeric(iris$Species)])

# 3d static
plot(bpca(iris[-5],
          d=1:3),
     var.color=c('blue', 'red'),
     var.cex=1,
     obj.names=FALSE,
     obj.cex=1,
     obj.col=c('red', 'green3', 'blue')[as.numeric(iris$Species)],
     obj.pch=c('+', '*', '-')[as.numeric(iris$Species)])

# 3d dynamic
plot(bpca(iris[-5],
          method='hj',
          d=1:3),
     rgl.use=TRUE,
     var.col=c('blue', 'red'),
     var.cex=1.2,
     obj.names=FALSE,
     obj.cex=.8,
     obj.col=c('red', 'green3', 'orange')[as.numeric(iris$Species)],
     simple.axes=FALSE,
     box=TRUE)
}

##
## New options plotting
##
plot(bpca(ontario))

# Labels for all objects
(obj.lab <- paste('g',
                  1:18,
                  sep=''))

# Giving obj.labels
plot(bpca(ontario),
    obj.labels=obj.lab) 

# Evaluate an object (1 is the default)
plot(bpca(ontario),
     type='eo',
     obj.cex=1)

plot(bpca(ontario),
     type='eo',
     obj.id=7,
     obj.cex=1)

# Giving obj.labels
plot(bpca(ontario),
     type='eo',
     obj.labels=obj.lab,
     obj.id=7,
     obj.cex=1)

# The same as above
plot(bpca(ontario),
     type='eo',
     obj.labels=obj.lab,
     obj.id='g7',
     obj.cex=1)

# Evaluate a variable (1 is the default)
plot(bpca(ontario),
     type='ev',
     var.pos=2,
     var.cex=1)

plot(bpca(ontario),
     type='ev',
     var.id='E7',
     obj.labels=obj.lab,
     var.pos=1,
     var.cex=1)

# A complete plot
cl <- 1:3
plot(bpca(iris[-5]),
     type='ev',
     var.id=1,
     obj.names=FALSE,
     obj.col=cl[as.numeric(iris$Species)])

legend('topleft',
       legend=levels(iris$Species),
       text.col=cl,
       pch=19,
       col=cl,
       cex=.9,
       box.lty=0)   

# Compare two objects (1 and 2 are the default)
plot(bpca(ontario),
     type='co')

plot(bpca(ontario),
     type='co',
     obj.labels=obj.lab)

plot(bpca(ontario),
     type='co',
     obj.labels=obj.lab,
     obj.id=13:14)

plot(bpca(ontario),
     type='co',
     obj.labels=obj.lab,
     obj.id=c('g7', 'g13'))

# Compare two variables
plot(bpca(ontario),
     type='cv')

# Which won where/what
plot(bpca(ontario),
     type='ww')

# Discrimitiveness vs. representativeness
plot(bpca(ontario),
     type='dv')

# Means vs. stability
plot(bpca(ontario),
     type='ms')

# Rank objects with ref. to the ideal variable 
plot(bpca(ontario),
     type='ro')

# Rank variables with ref. to the ideal object
plot(bpca(ontario),
     type='rv')

\dontrun{
plot(bpca(iris[-5]),
     type='eo',
     obj.id=42,
     obj.cex=1)

plot(bpca(iris[-5]),
     type='ev',
     var.id='Sepal.Width')

plot(bpca(iris[-5]),
     type='ev',
     var.id='Sepal.Width',
}

devAskNewPage(oask)
}

\name{gabriel1971}
\alias{gabriel1971}

\title{
  Percentages of households having various facilities and appliances in East
  Jerusalem Arab areas, by quarters of the town
}

\description{
  Percentages of households having various facilities and appliances in East
  Jerusalem Arab areas, by quarters of the town. The average percentages in
  each quarter indicate the standard of living of that area and the average
  percentage of each facility or appliance its over-all prevalence.
}

\usage{
  data(gabriel1971)
}

\format{
  The format is:
  \verb{
    num [1:8, 1:9] 98.2 78.8 14.4 86.2 32.9 73 4.6 29.2 97.2 81 ...
    - attr(*, "dimnames")=List of 2
    ..$ : chr [1:8] "toilet" "kitchen" "bath" "eletricity" ...
    ..$ : chr [1:9] "CRISTIAN" "ARMENIAN" "JEWISH" "MOSLEM" ...
  }
}

\references{
  Gabriel, K. R.  (1971)
  The biplot graphical display of matrices with application to principal component analysis.
  \emph{Biometrika} \bold{58}, 453-467.
}

\examples{
##
## A simple example
##
data(gabriel1971)
bp <- bpca(gabriel1971)

dev.new(w=6, h=6)
plot(bp)

# Exploring the object 'bp' created by the function 'bpca'
class(bp)
names(bp)
str(bp)

summary(bp)
bp$call
bp$eigenval
bp$eigenvec
bp$numb
bp$import
bp$coord
bp$coord$obj
bp$coord$var
bp$var.rb
bp$var.rd
}

\keyword{datasets}

\name{var.rbf}
\alias{var.rbf}

\title{
  Projected Correlations by bpca
}

\description{
  Computes the matrix of graphical correlations represented by biplot
  for a matrix of variable coordinates.
}

\usage{
  var.rbf(x)
}

\arguments{
  \item{x}{A given object of the classes \samp{bpca.2d} and \samp{bpca.3d}.}
}

\value{
  A \code{matrix} of graphical correlations represented by biplot.
}

\note{
  This function is mainly for internal use in the \pkg{bpca} package,
  and may not remain available (unless we see a good reason).
}

\author{
  Faria, J. C. \cr
  Allaman, I. B. \cr
  Demétrio C. G. B.
}

\seealso{\code{\link{bpca}}}

\examples{
##
## Particular use
##

bp1 <- bpca(gabriel1971)
bp1$var.rb # NA

# Computes the correlations of all variables under the biplot projection
(res <- var.rbf(bp1$coord$var)) 

##
## Common use
##

bp2 <- bpca(gabriel1971,
            var.rb=TRUE)

bp2$var.rb
}

\keyword{multivariate}

\name{summary.bpca}
\alias{summary.bpca}

\title{
  Summary Method for bpca Objects
}

\description{
  Returns (and prints) a summary list for \code{bpca} (\code{bpca.2d} and 
  \code{bpca.3d}) objects.
}

\usage{
  \method{summary}{bpca}(object,
        presentation=FALSE, \dots)
}

\arguments{
  \item{object}{A given object of the class \code{bpca}, (\code{bpca.2d} or \code{bpca.3d}).}
  \item{presentation}{Logic.
    If \code{TRUE} the summary of the class \code{bpca}, (\code{bpca.2d}, \code{bpca.3d} is showed well formated in the screen, else, return a list.
    The default is \code{FALSE}.}
  \item{\dots}{Potential further arguments (require by generic).}
}

\author{
  Faria, J. C. \cr
  Allaman, I. B. \cr
  Demétrio C. G. B.
}

\seealso{
  \code{\link{bpca}},
  \code{\link{xtable.bpca}}
}

\examples{
##
## Example 1
## bpca - 2d
##
# bpca
bp <- bpca(gabriel1971)
summary(bp)
summary(bp,
        presentation=TRUE)

##
## Example 2
## bpca - 3d
##
bp <- bpca(gabriel1971,
           d=1:3)

# bpca
sm <- summary(bp)
str(sm)
sm
summary(bp,
        presentation=TRUE)
}

\keyword{bpca}
\keyword{summary}
\keyword{multivariate}

\name{gge2003}
\alias{gge2003}

\title{
  A didactic matrix of genotypes (rows) and environments (columns)
}

\description{
  A didatic matrix of genotypes (rows) and environments (columns) proposed by
  Weikai Yan and Manjit S. Kang in GGE biplot analysis: a graphical tool for
  breeders, geneticists, and agronomists.
}

\usage{
  data(gge2003)
}

\format{
  The format is:
  \verb{
    num [1:4, 1:3] 20 6 -10 8 -9 12 -6 -12 6 -15 ...
    - attr(*, "dimnames")=List of 2
    ..$ : chr [1:4] "G1" "G2" "G3" "G4"
    ..$ : chr [1:3] "E1" "E2" "E3"
  }
}

\references{
  Yan, B. W. and Kang, M. S. (2003)
  \emph{GGE biplot analysis : a graphical tool for breeders,
    geneticists, and agronomists}. CRC Press, New York, 288p.
}

\examples{
##
## Example from YAN, W & KANG, M.S. GGE biplot analysis : a graphical tool
## for breeders, geneticists, and agronomists
##

data(gge2003)
bp <- bpca(t(gge2003), var.rb=TRUE)

as.dist(bp$var.rb)

dev.new(w=8, h=4)
op = par(no.readonly=TRUE)
par(mfrow=c(1,2))

plot(bpca(gge2003, var.pos=2),
     main='Columns as variables \n (var.pos=2)',
     var.col=1,
     obj.col=c(2:4, 2), obj.cex=.8)

plot(bpca(gge2003, var.pos=1),
     main='Rows as variables \n (var.pos=1)',
     var.col=1,
     obj.col=2:4, obj.cex=.8)

par(op)
}

\keyword{datasets}

\name{bpca}
\alias{bpca}
\alias{bpca.default}
\alias{bpca.prcomp}

\title{
  Biplot of Multivariate Data Based on Principal Components Analysis
}

\description{
  Computes biplot reduction on \code{data.frame}, \code{matrix} or
  \code{prcomp} objects and returns a \code{bpca} object.
}

\usage{
  bpca(x, \dots)
  \method{bpca}{default}(x,
     d=1:2,
     center=2,
     scale=TRUE,
     method=c('hj', 'sqrt', 'jk', 'gh'),
     iec=FALSE,
     var.rb=FALSE,
     var.rd=FALSE,
     limit=10, \dots)
  \method{bpca}{prcomp}(x,
     d=1:2, \dots)
}

\arguments{
  \item{x}{A \code{data.frame}, \code{matrix} or \code{prcomp} object.}
  \item{d}{A vector giving the first and last eigenvalue to be considered by the biplot reduction.
    It can be \code{d=1:3} or \code{d=c(1,3)} for 3d biplot.
    The default is \code{d=1:2}.}
  \item{center}{Numeric.
    The type of centering to be performed: \cr
    \samp{0} - no centering \cr
    \samp{1} - global-centered = \code{sweep(x, 1, mean(x))} \cr
    \samp{2} - column-centered = \code{sweep(x, 2, apply(x, 2, mean))} \cr
    \samp{3} - double-centered = \code{sweep(sweep(x, 1, apply(x, 1, mean)),
                                             2, apply(x, 2, mean)) + mean(x)} \cr
    The default is 2.}
  \item{scale}{Logical.
    A value indicating whether the variables should be
    scaled to have unit variance before the analysis takes place: \cr
    \code{FALSE} - no scale; \cr
    \code{TRUE} - scale.}
  \item{method}{A vector of character strings that indicates the method of factorization: \cr
    \samp{hj} - \samp{HJ} (\samp{simetric}, Galindo Villardón (1986)); \cr
    \samp{sqrt} - \samp{SQRT} (\samp{squared root - simetric}, Gabriel (1971)); \cr
    \samp{jk} - \samp{JK} (\samp{row metric preserving}, Gabriel (1971)); \cr
    \samp{gh} - \samp{GH} (\samp{column metric preserving}, Gabriel (1971)).}
  \item{iec}{Logical.
    If \code{TRUE} the matrix of eigenvalues, coordinates od objects and variables will be inverted.
    The default is \code{FALSE}.}
  \item{var.rb}{A logical value.
    If \code{TRUE}, all correlation coefficients for all variables (\emph{under the biplot projection}) will be computed.}
  \item{var.rd}{A logical value.
    If \code{TRUE}, the diagnostic of the representation of variables projected by the biplot will be computed.
    If \code{var.rd} is \code{TRUE} the \code{var.rb} parameter must be also \code{TRUE}.}
  \item{limit}{A vector giving the percentual limit to define poor representation of variables.}
  \item{\dots}{Additional parameters.
    It is necessary to be S3 method.}
}

\details{
  The biplot is a multivariate method for graphing row and column elements using
  a single plot (Gabriel, 1971).

  The biplot of a matrix
  \deqn{_{n}Y_{p}}
  projects on the same plot: \emph{rows}
  (associated with \emph{n} objects) and \emph{columns} (associated with
  \emph{p} variables), markers from data that forms a two-way table
  (\code{data.frame} or \code{matrix} object).
  The markers are computed from the singular value decomposition,
  \code{svd(Y)}, and subsequent factorization.

  The \emph{bi} refers to the kind of information contained in a data set
  disposed in a two-way table. If the data are a \emph{tri}-dimensional array
  the method will be called \emph{triplot} (not still contemplated in the bpca
  package).

  The basic idea behind the biplot method was to add the information about the
  variables to the principal component graph (Johnson & Wichern, 1988).

  Considering the results of
  \deqn{svd(_{n}Y_{p})}
  \itemize{
    \item \emph{d}: A vector containing the singular values of Y, of length
    \code{min(n, p)};
    \item \emph{u}: A matrix whose columns contain the left singular vectors of
    Y, present if \samp{nu > 0}. Dimension \samp{c(n, nu)};
    \item \emph{v}: A matrix whose columns contain the right singular vectors of
    Y, present if \samp{nv > 0}. Dimension \code{c(p, nv)}.
  }  

  and also,
  \deqn{s^2 = diag(d)}
  \deqn{n = n\_objects(Y)}

  it is possible an approximation of Y:
  \deqn{_{n}Y_{p} \approx Y_{m} = g.h'}

  in various ways. The methods of factorization computed by the \code{bpca}
  function are:
  \itemize{
    \item \emph{HJ - simetric}, Galindo Villardón (1986):
    \deqn{g = u*s^2}
    \deqn{h = s^2*v'}
    \item \emph{SQRT - squared root simetric}, Gabriel (1971):
    \deqn{g = u*\sqrt{s^2}}
    \deqn{h = \sqrt{s^2}*v'}
    \item \emph{JK - row metric preserving}, Gabriel (1971):
    \deqn{g = u*s^2}
    \deqn{h = v'}
    \item \emph{GH - column metric preserving}, Gabriel (1971):
    \deqn{g = \sqrt{n-1}*u}
    \deqn{h = \frac{1}{\sqrt{n-1}}*s^2*v'}
  }  

  Considering
  \deqn{_{n}Y_{p} \approx Y_{m}}
  it is possible to deduce that if the
  rank (r) of the matrix
  \deqn{_{n}Y_{p}}
  is bigger than `m', the biplot
  representation of Y will be an approximation, and accurate only in the case
  of \deqn{r = m}.

  Due to the need of different methods of factorization, if \samp{x} is a
  \code{prcomp} object, the method \code{bpca.prcomp} will go back from the
  \code{prcomp} function. In other words, it will regenerates, or computes, the
  inverse of the \code{svd} decomposition of the given data
  \deqn{_{n}Y_{p}}
  After this, it will make a call to the method \code{bpca.default} with the
  adequate parameters.

  The biplot is used with many multivariate methods to display
  relationships between objects, variables and the interrelationship
  between objects and variables (as prevalence, importance). There are many
  variations of biplots (see the references).
}

\value{
  The function \code{bpca} returns an object of class \code{bpca.2d} or
  \code{bpca.3d}. Both are \code{list} objects with the slots:
  \item{call}{The call used.}
  \item{eigenvalues}{A vector of the eigenvalues.}
  \item{eigenvectors}{A vector of the eigenvectors.}
  \item{numer}{A vector of the number of eigenvalues considered in the
        reduction.}
  \item{importance}{A matrix with the \emph{general} and \emph{partial}
        variation explained by the reduction.}
  \item{coord}{A list with the coordinates of the two components: objects
        and variables.}
  \item{var.rb}{A matrix of all correlation coefficients for all variables
        under the biplot projection.}
  \item{var.rd}{A matrix of the diagnostic of the poor projection of variable
        correlations by the biplot reduction.}
}

\author{
  Faria, J. C. \cr
  Allaman, I. B. \cr
  Demétrio C. G. B.
}

\references{
  Gabriel, K. R.  (1971)
  The biplot graphical display of matrices with application to principal
  component analysis.
  \emph{Biometrika} \bold{58}, 453-467.

  Galindo Vilardón, M. P. (1986)
  Una alternativa de representación simultánea: HJ-Biplot.
  \emph{Qüestiió}, 10(1):13-23, 1986.

  Johnson, R. A. and Wichern, D. W. (1988)
  \emph{Applied multivariate statistical analysis}.
  Prentice-Hall, Inc., Upper Saddle River, NJ, USA, 6 ed.

  Gower, J.C. and Hand, D. J.  (1996)
  \emph{Biplots}. Chapman & Hall.

  Yan, B. W. and Kang, M. S. (2003)
  \emph{GGE biplot analysis: a graphical tool for breeders,
  geneticists, and agronomists}. CRC Press, New York, 288p.
}

\seealso{\code{\link{biplot.prcomp}}}

\examples{
##
## Example 1
## Computing and ploting a bpca object with 'graphics' package - 2d
##

bp <- bpca(gabriel1971)

dev.new(w=6, h=6)
oask <- devAskNewPage(dev.interactive(orNone=TRUE))
plot(bp)

# Exploring the object 'bp' created by the function 'bpca'
class(bp)
names(bp)
str(bp)

summary(bp)
bp$call
bp$eigenval
bp$eigenvec
bp$numb
bp$import
bp$coord
bp$coord$obj
bp$coord$var
bp$var.rb
bp$var.rd

\dontrun{
##
## Example 2
## Computing and plotting a bpca object with 'scatterplot3d' package - 3d
##

bp <- bpca(gabriel1971,
           d=2:4)

plot(bp)

# Exploring the object 'bp' created by the function 'bpca'
class(bp)
names(bp)
str(bp)

summary(bp)
bp$call
bp$eigenval
bp$eigenvec
bp$numb
bp$import
bp$coord
bp$coord$obj
bp$coord$var
bp$var.rb
bp$var.rd

##
## Example 3
## Computing and plotting a bpca object with 'rgl' package - 3d
##

plot(bpca(gabriel1971,
          d=1:3),
     rgl.use=TRUE)

# Suggestion: Interact with the graphic with the mouse
# left button: press, maintain and movement it to interactive rotation;
# right button: press, maintain and movement it to interactive zoom.
# Enjoy it!

##
## Example 4
## Grouping objects with different symbols and colors - 2d and 3d
##

# 2d
plot(bpca(iris[-5]),
     var.cex=.7,
     obj.names=FALSE,
     obj.cex=1.5,
     obj.col=c('red', 'green3', 'blue')[as.numeric(iris$Species)],
     obj.pch=c('+', '*', '-')[as.numeric(iris$Species)])

# 3d static
plot(bpca(iris[-5],
          d=1:3),
     var.color=c('blue', 'red'),
     var.cex=1,
     obj.names=FALSE,
     obj.cex=1,
     obj.col=c('red', 'green3', 'blue')[as.numeric(iris$Species)],
     obj.pch=c('+', '*', '-')[as.numeric(iris$Species)])

# 3d dynamic
plot(bpca(iris[-5],
          method='hj',
          d=1:3),
     rgl.use=TRUE,
     var.col=c('blue', 'red'),
     var.cex=1.2,
     obj.names=FALSE,
     obj.cex=.8,
     obj.col=c('red', 'green3', 'orange')[as.numeric(iris$Species)],
     simple.axes=FALSE,
     box=TRUE)
}

devAskNewPage(oask)       
}

\keyword{multivariate}

\name{print.xtable}
\alias{print.xtable}
\alias{print.xtable.bpca}
\alias{print.xtable.default}

\title{
  Print Method for xtable.bpca Objects
}

\description{
  Returns (and prints) a summary list for \code{xtable.bpca} objects.
}

\usage{
\method{print}{xtable.bpca}(x, 
      hline.after = getOption("xtable.hline.after", NULL), 
      include.colnames = getOption("xtable.include.colnames", FALSE), 
      add.to.row = getOption("xtable.add.to.row", NULL), 
      sanitize.text.function = getOption("xtable.sanitize.text.function", NULL),
      sanitize.rownames.function = getOption("xtable.sanitize.rownames.function", 
                                             sanitize.text.function), 
      sanitize.colnames.function = getOption("xtable.sanitize.rownames.function", 
                                             sanitize.text.function),\dots)
}

\arguments{
  \item{x}{A given object of the class \code{xtable} or \code{xtable.bpca}.}
  \item{hline.after}{When type="latex", a vector of numbers between -1 and nrow(x), 
    inclusive, indicating the rows after which a horizontal line should appear.
    Default value is NULL which means draw a line before and after the columns names, 
    draw a line before variables and at the end of the table.}
  \item{include.colnames}{If TRUE the columns names are printed.
    Default value is FALSE which means a column more elaborate was done.} 
  \item{add.to.row}{A list of two components. The first component (which should be called 'pos') 
    is a list that contains the position of rows on which extra commands should be added at the end.
    The second component (which should be called 'command') is a character vector of the same length
    as the first component, which contains the command that should be added at the end of the specified rows. 
    Default value is NULL.}
  \item{sanitize.text.function}{All non-numeric entries (except row and column names) are sanitized 
    in an attempt to remove characters which have special meaning for the output format. 
    If sanitize.text.function is not NULL, it should be a function taking a character vector and returning one,
    and will be used for the sanitization instead of the default internal function. Default value is NULL.}
  \item{sanitize.rownames.function}{Like the sanitize.text.function, but applicable to row names. 
    The default uses the sanitize.text.function.}
  \item{sanitize.colnames.function}{Like the sanitize.text.function, but applicable to column names. 
    The default uses the sanitize.text.function.}
  \item{\dots}{Other arguments of the print.xtable function (see xtable package).}
}

\author{
  Faria, J. C. \cr
  Allaman, I. B. \cr
  Demétrio C. G. B.
}

%\seealso{
%  \code{\link{bpca}},
%  \code{\link[xtable]{xtable}}
%}

\examples{

## Example 1: Principal labels in portuguese
library(xtable)

bp2 <- bpca(gabriel1971)  
tbl <- xtable(bp2)
rownames(tbl) <- gsub('Eigenvectors','Autovetores',rownames(tbl))
rownames(tbl) <- c(rownames(tbl)[1:9],'Autovalores','Variância retida','Variância acumulada')
dimnames(tbl)[[2]] <- c('CP 1','CP 2')

print(tbl)

## Example 2: With bold in the column  
tbl1 <- xtable(bp2)
bold <- function(x){
  paste('\\textbf{',
        x, 
        '}')
}

print(tbl1,
      sanitize.colnames.function = bold)

# Example 3: With italic in the rows
tbl2 <- xtable(bp2)
italic <- function(x){
  paste('& \\textit{',
        x, 
        '}')
} # It is necessary the character "&" to adapt the number of column of the table!

print(tbl2,
      sanitize.rownames.function = italic)

}

\keyword{multivariate}

\name{ontario}
\alias{ontario}

\title{
  Ontario winter wheat (1993)
}

\description{
  The sample data are yields from the 1993 Ontario winter wheat (Triticum aestivum L.) performance
  trials, in which 18 cultivars were tested at nine locations (Yan and Kang 2003)
}

\usage{
  data(ontario)
}

\format{
  A data frame with 18 observations on the following 10 variables.
}

\source{
  Yan W, Kang MS (2003). GGE Biplot Analysis: A Graphical Tool for Breeders, Geneticists,
  and Agronomists. CRC Press, Boca Raton, FL, USA.
}

\examples{
data(ontario)

# 2d
plot(bpca(ontario,
          d=1:2))

# 3d
plot(bpca(ontario,
          d=1:3),
     rgl.use=TRUE)
}

\keyword{datasets}

\name{dt.tools}
\alias{dt.tools}

\title{
  Data Tools for Multivariate
}

\description{
  Calculates vector variable lengths, angles between vector variables and
  variable correlations from `data.frame' or `matrix' objects
}

\usage{
  dt.tools(x,
           center=2,
           scale=TRUE)
}

\arguments{
  \item{x}{A \code{data.frame} or \code{matrix} object.}
  \item{center}{Numeric.
    The type of centering to be performed: \cr
    \samp{0} - no centering; \cr
    \samp{1} - global-centered = \code{sweep(x, 1, mean(x))}; \cr
    \samp{2} - column-centered = \code{sweep(x, 2, apply(x, 2, mean))}; \cr
    \samp{3} - double-centered = \code{sweep(sweep(x, 1, apply(x, 1, mean)),
                                             2, apply(x, 2, mean)) + mean(x)}. \cr
    The default is 2.}
  \item{scale}{Logical.
    A value indicating whether the variables should be scaled to have unit
    variance before the analysis takes place: \cr
    \code{FALSE} - no scale; \cr
    \code{TRUE} - scale.}
}

\details{
  This function computes: vector variable lengths, angles between vector
  variables and variable correlations from \code{data.frame} or \code{matrix} objects.

  If the data are centered (\code{center=2}), the correlations are the
  same as those obtained by the \code{cor} function.
}

\value{
  An \code{list} with the components:
  \item{length}{A vector of the lengths.}
  \item{angle}{A matrix of the angles.}
  \item{r}{A matrix of the observed correlations.}
}

\author{
  Faria, J. C. \cr
  Allaman, I. B. \cr
  Demétrio C. G. B.
}

\references{
  Johnson, R. A. and Wichern, D. W. (1988)
  \emph{Applied multivariate statistical analysis}.
  Prentice-Hall, Inc., Upper Saddle River, NJ, USA, 6 ed.
}

\seealso{\code{\link{bpca}}}

\examples{
##
## Computes: vector variable lengths, angles between vector variables and
## variable correlations from data.frame or matrix objects (n x p)
## n = rows (objects)
## p = columns (variables)
##

dt <- dt.tools(iris,
               2)  # No numeric columns are removed in dt.tools 

# Exploring the object 'bp' created by the function 'var.tools'
class(dt)
names(dt)
str(dt)

dt$length
dt$angle
dt$r
dt

# Checking the determinations
(iris.tools <- round(dt.tools(iris,
                              center=2)$r,
                     5))

(iris.obsv  <- round(cor(iris[-5]),
                     5))

all(iris.tools == iris.obsv)
}

\keyword{multivariate}

\name{xtable.bpca}
\alias{xtable.bpca}

\title{
  LaTeX table of the Biplot.
}

\description{
  This function returns a LaTeX table of the bpca objects.
}

\usage{
\method{xtable}{bpca}(x,
       caption = NULL,
       label = NULL,
       align = NULL,
       digits = NULL,
       display = NULL,
       auto = FALSE, \dots)
}

\arguments{
  \item{x}{A object of the class \code{\link{bpca}} objects.}
  \item{caption}{Character vector of length 1 or 2 containing the table's
    caption or title. If length is 2, the second item is the "short
    caption" used when LaTeX generates a "List of Tables".
    Set to NULL to suppress the caption. Default value is \code{NULL}.}
  \item{label}{Character vector of length 1 containing the
    LaTeX \samp{\\label} or HTML anchor. Set to \code{NULL} to suppress
    the label. Default value is \code{NULL}.}
  \item{align}{Character vector of length equal to the number of columns of
    the resulting table, indicating the alignment of the corresponding
    columns. Also, "|" may be used to produce vertical lines between
    columns in LaTeX tables, but these are effectively ignored when
    considering the required length of the supplied vector.
    If a character vector of length one is supplied, it is split as
    strsplit(align, "")[[1]] before processing. Since the row names
    are printed in the first column, the length of align is one greater
    than ncol(x) if x is a data.frame. Use "l", "r", and "c" to denote
    left, right, and center alignment, respectively. for a LaTeX column
    of the specified width. For HTML output the "p" alignment is interpreted
    as "l", ignoring the width request.
    Default depends on the class of x.}
  \item{digits}{Numeric vector of length equal to one (in which case it
    will be replicated as necessary) or to the number of columns of
    the resulting \code{table} or \code{matrix} of the same size as
    the resulting table, indicating the number of digits to display
    in the corresponding columns. Since the row names are printed
    in the first column, the length of the vector digits or the number
    of columns of the matrix digits is one greater than ncol(x) if x
    is a \code{data.frame}. Default depends on the class of x.
    If values of digits are negative, the corresponding values of x
    are displayed in scientific format with abs(digits) digits.}
  \item{display}{Character vector of length equal to the number of columns
    of the resulting table, indicating the format for the corresponding
    columns. Since the row names are printed in the first column, the
    length of display is one greater than ncol(x) if x is a data.frame.
    These values are passed to the formatC function. Use "d" (for integers),
    "f", "e", "E", "g", "G", "fg" (for reals), or "s" (for strings).
    "f" gives numbers in the usual xxx.xxx format; "e" and "E" give
    n.ddde+nn or n.dddE+nn (scientific format); "g" and "G" put x[i] into
    scientific format only if it saves space to do so. "fg" uses fixed
    format as "f", but digits as number of significant digits. Note that
    this can lead to quite long result strings.
    Default depends on the class of x.}
  \item{auto}{Logical, indicating whether to apply automatic format when no
    value is passed to align, digits, or display. This autoformat
    (based on xalign, xdigits, and xdisplay) can be useful to quickly
    format a typical \code{matrix} or \code{data.frame}.
    Default value is \code{FALSE}.}
  \item{\dots}{Additional arguments. (Currently ignored.)}
}

\details{
  This function extracts tabular information from x and returns an object of class "xtable.bpca", "xtable" or "data.frame". 

  It is necessary to declare the latex packages: \samp{multirow} in the preamble of the Rnoweb file to make available all the 
  resources of the function \code{xtable.bpca}.
}

\value{
  An object of the class \code{xtable.bpca}.
}

\author{
  Faria, J. C. \cr
  Allaman, I. B. \cr
  Demétrio C. G. B.
}

\references{
  Johnson, R. A. and Wichern, D. W. (1988)
  \emph{Applied multivariate statistical analysis}.
  Prentice-Hall, Inc., Upper Saddle River, NJ, USA, 6 ed.
}

%\seealso{
%  \code{\link{bpca}},
%  \code{\link[xtable]{xtable}}
%}

\examples{
## Example 1: The simplest possible 
library(xtable)

bp <- bpca(iris[-5],
           d=1:3)

xtable(bp)

## Example 2: With caption and label 
bp2 <- bpca(gabriel1971) 

xtable(bp2,
       caption='Biplot gabriel1971',
       label='example_2')
}

\keyword{multivariate}
\keyword{table}
\keyword{latex}
\keyword{bpca}

\name{qbpca}
\alias{qbpca}

\title{
  Quality of the Representation of Variables by Biplot
}

\description{
  This function returns an object of the class \code{qbpca}. It is a simple
  measure of the quality of biplot representation of the variables. The
  observed (in the data) and projected (under biplot reduction) correlations
  are computed.
}

\usage{
  qbpca(x,
        bpca)
}

\arguments{
  \item{x}{A \code{data.frame} or \code{matrix} object.}
  \item{bpca}{A object of the class \code{\link{bpca}}.}
}

\details{
  This function binds the vectors of observed (from the matrix or data.frame)
  and projected (under biplot reduction) correlations for all variables.
}

\value{
  An object of class \code{qbpca} and \code{data.frame} with two columns:
  \item{obs}{A vector of the observed correlations for all variables.}
  \item{var.rb}{A vector of the projected correlations for all variables
    determined under biplot reduction).}
}

\author{
  Faria, J. C. \cr
  Allaman, I. B. \cr
  Demétrio C. G. B.
}

\references{
  Johnson, R. A. and Wichern, D. W. (1988)
  \emph{Applied multivariate statistical analysis}.
  Prentice-Hall, Inc., Upper Saddle River, NJ, USA, 6 ed.
}

\seealso{\code{\link{bpca}}}

\examples{
##
## Example 1
## Example of 'var.rb=TRUE' parameter as a measure of the quality of the biplot - 2d
##

oask <- devAskNewPage(dev.interactive(orNone=TRUE))

## Differences between methods of factorization
# SQRT
bp1 <- bpca(gabriel1971,
            meth='sqrt',
            var.rb=TRUE)

qbp1 <- qbpca(gabriel1971,
              bp1)

plot(qbp1,
     main='sqrt - 2d \n (poor)')


# JK
bp2 <- bpca(gabriel1971,
            meth='jk',
            var.rb=TRUE)

qbp2 <- qbpca(gabriel1971,
              bp2)

plot(qbp2,
     main='jk - 2d \n (very poor)')


# GH
bp3 <- bpca(gabriel1971,
            meth='gh',
            var.rb=TRUE)

qbp3 <- qbpca(gabriel1971,
              bp3)

plot(qbp3,
     main='gh - 2d \n (good)')


# HJ
bp4 <- bpca(gabriel1971,
            meth='hj',
            var.rb=TRUE)

qbp4 <- qbpca(gabriel1971,
             bp4)

plot(qbp4,
     main='hj - 2d \n (good)')

##
## Example 2
## Example of 'var.rb=TRUE' parameter as a measure of the quality of the biplot - 3d
##

## Differences between methods of factorization
# SQRT
bp1 <- bpca(gabriel1971,
            meth='sqrt',
            d=1:3,
            var.rb=TRUE)

qbp1 <- qbpca(gabriel1971,
              bp1)

plot(qbp1,
     main='sqrt - 3d \n (poor)')


# JK
bp2 <- bpca(gabriel1971,
            meth='jk',
            d=1:3,
            var.rb=TRUE)

qbp2 <- qbpca(gabriel1971,
             bp2)

plot(qbp2,
     main='jk - 3d \n (very poor)')


# GH
bp3 <- bpca(gabriel1971,
            meth='gh',
            d=1:3,
            var.rb=TRUE)

qbp3 <- qbpca(gabriel1971,
              bp3)

plot(qbp3,
     main='gh - 3d \n (whow!)')


# HJ
bp4 <- bpca(gabriel1971,
            meth='hj',
            d=1:3,
            var.rb=TRUE)

qbp4 <- qbpca(gabriel1971,
              bp4)

plot(qbp4,
     main='hj - 3d \n (whow!)')

devAskNewPage(oask)  
}

\keyword{multivariate}
